/**
 * Created by JSmith on 14/6/2016.
 *
 * This component can be used to use an analytics visualization as an action on the case display page
 *
 * It provides:
 *
 * display of visualization
 * ability to send a filter in to the visualization based on case data
 * ability to mark data inside the visualization then pass it back to a form (via $scope)
 *
 * uses config.json for params:
 *
 *     "_analyticsActionConfig": "This component lets you mark data inside a vizualization and pass it to a BPM form. It will also filter the vizualization based on a filter",
 "analyticsActionConfig": {
        "_actionName": "The name of the action which you are using for analytics",
        "actionName" : "Analyze",
        "_tableName": "This is the name of the table inside the vizualization on which you will mark data",
        "tableName": "Customer Transactions Tall",
        "_analysisPath": "The path to the vizualization to display",
        "analysisPath": "/AntiMoneyLaundering/Anti_Money_Laundering_Template",
        "_markingName": "This is the name of the marking you are using in the vizualization",
        "markingName": "markedTransactions",
        "_caseIdField": "Not used",
        "caseIdField": "",
        "_columnNames": "These are the marking columns passed by your marking to BPM - order should be the same as targetFormClassAttributes",
        "columnNames": [
            "txn_type",
            "bank_id",
            "country_id",
            "acc_num"
        ],
        "_targetFormFieldName" : "This is the array field in your form to which you are adding marking data (one array element per marking)",
        "targetFormFieldName" : "transactions",
        "_targetFormClass": "This is the BOM class of the targetFormField",
        "targetFormClass": "com_tibco_tms_casemodel",
        "_targetFormClassConstructor": "This is the constructor method to create the targetFormClassField array element - typically create<bomClassName>",
        "targetFormClassConstructor": "createTransactions",
        "_targetFormClassAttributes": "These are the attributes inside the targetFormField which you are populating from the data in columnNames - the order should be the same - and they should all be text or numeric - use text for dates",
        "targetFormClassAttributes": [
            "TransactionType",
            "OriginatorBankId",
            "OriginatorCountry",
            "OriginatorAccountNumber"
        ],
        "_casetype": "the data type you are marking",
        "casetype": "transactions",
        "_spotfireServer": "Spotfire Web Player URL",
        "spotfireServer": "http://52.19.185.101:28080/spotfire/wp",
        "_bdsVersion": "Not used",
        "bdsVersion": "-1",
        "_filterTable": "This is the table on which a filter is applied",
        "filterTable": "All Customer Relationships",
        "_filterScheme": "The filter scheme",
        "filterScheme": "Customer Reference and Transaction Link Analysis",
        "_filterColumn": "The column on which you want to filter",
        "filterColumn": "Customer1",
        "_filterValue": "The value(s) on which you are filtering",
        "filterValue": "customerName"
    },
 *
 */

angular.module('components.analyticsAction', [])
    .controller('AnalyticsActionController', ['$scope', '$rootScope', "$timeout", "$location", function ($scope, $rootScope, $timeout, $location) {

		//console.log("*****************AnalyticsAction Controller");
		$scope.editMode = false;
		// callback for webplayer marking directive
		$scope.visualCaseMarking = function(selection_size, selection) {
			//console.log("Analytics action got callback - setting scope");
			$scope.marking_size=selection_size;
			$scope.marking = selection;
			$scope.$apply();
		};

		// this is called when you click the ADD button - it will attempt to copy the data from the $scope into a form field
		$scope.addActionsToCase = function() {
			//console.log("Adding fieldObj: " + $scope.marking);
			for (i=0;i<$scope.marking_size;i++) {
				var theMarking = $scope.marking[i];
				var fieldObj = $rootScope.analyticsActionHandler.form.a._factory[$scope.analyticsActionConfig.targetFormClass][$scope.analyticsActionConfig.targetFormClassConstructor]();

				// loop through the markings, and copy to matching field on the form
				for (f=0;f<$scope.analyticsActionConfig.columnNames.length;f++) {
						var fieldNameMethod = "set" + $scope.analyticsActionConfig.targetFormClassAttributes[f];
						var fieldValue = theMarking[$scope.analyticsActionConfig.columnNames[f]];
						fieldObj[fieldNameMethod](fieldValue);
				}

				$rootScope.analyticsActionHandler.form.a.p[$scope.analyticsActionConfig.targetFormFieldName].add(fieldObj);
			}

			// now run the submit action
			// for the submit action the second parameter doesn't really matter - if this were a script, the second parameter would be 'this'
			// so you you would want to pass something more useful
			$rootScope.analyticsActionHandler.form.a.invokeAction('submit', $rootScope.analyticsActionHandler.form.a.controlList);
			//console.log("*&* Added marked data to form and submitted");
		}

		// not used - for future
		$scope.toggleEditMode = function() {
			//console.log("toggle edit mode");
			$scope.editMode = !$scope.editMode;
		}
	}]);