/**
 * Created by JSmith on 3/18/2015.
 */

angular.module('components.caseDisplay', ['components.bpmCaseDisplay', 'components.tableDisplay.directive', 'components.tableDisplay','components.linkedCaseDisplay.directive', 'components.linkedCaseDisplay', 'components.caseClassDisplay.directive', 'components.caseDocumentDisplay.directive', 'components.caseCommentDisplay.directive', 'components.caseUrlDisplay.directive', 'components.caseTaskDisplay.directive', 'components.caseWorkitemDisplay.directive', 'components.caseMilestone.directive', 'components.caseClassDisplay', 'components.caseContextMenu', 'components.caseContextMenu.directive', 'components.caseActionList.directive', 'angularUtils.directives.dirPagination', 'components.microService.directive', 'components.caseAuditDisplay.directive', 'components.casePendingList.directive', 'components.caseBsDisplay.directive', 'components.caseSpotfireDisplay.directive'])
    .controller('CaseDisplayController', ["BPMWorklistService", "BPMBusinessActionsService", "BPMLoginService", "CaseInformationService", "CaseDisplayBPMServices", "$mdDialog", "$timeout", "$location", "$rootScope", "$http", "$anchorScroll", "$mdMedia", "$scope", function (BPMWorklistService, BPMBusinessActionsService, BPMLoginService, CaseInformationService, CaseDisplayBPMServices, $mdDialog, $timeout, $location, $rootScope, $http, $anchorScroll, $mdMedia, $scope) {

		// We will set:
		// caseDisplay.caseinfo (case data)
		// caseDisplay.caseinfo.caseRef
		// caseDisplay.caseinfo.caseReference
		// caseDisplay.caseinfo.caseDataTree (the actual case data tree)
		// caseDisplay.caseinfo.casedata (the top (first) level class data of caseDataTree in a more accessible format - eg
		// {"id":"304","status":"INVESTIGATION","received":"2016-02-02T12:03:43Z","target":"2016-02-08T12:03:48Z"})
		// caseDisplay.caseinfo.caseDataAll (this is all case data but translated to a cleaner structure for template marking)
		// caseDisplay.summarydata (case summary data)
		// caseDisplay.caseinfo.state (case state)
		// caseDisplay.caseinfo.stateFieldName (stateFieldName)
		// caseDisplay.caseinfo.stateFieldLabel (stateFieldLabel)
		// caseDisplay.caseinfo.caseId (caseId)
		// caseDisplay.caseinfo.caseIdFieldName (caseIdFieldName)
		// caseDisplay.caseinfo.caseIdFieldLabel (caseIdFieldLabel)
		// caseDisplay.caseinfo.states (possible states in order)
		// caseDisplay.caseinfo.caseClass (full case class - like 'com.example.complaint')
		// caseDisplay.caseinfo.caseType (case type - short class name like 'complaint')
		// caseDisplay.caseinfo.linkedCaseDetails (all the cases linked to this case)
		// caseDisplay.casemodels (All deployed case data model details, includes all attributes and ENUMs - eg. for states)
		// caseDisplay.caseinfo.casemodel (The case model for this case class)
		// caseDisplay.caseinfo.caseAttributeLabels (Each top level attributes label) - note these fields must be marked as searchable in the data model
		// caseDisplay.dataloaded (set to true when all case data is loaded)
		// caseDisplay.userName (current username)
		// caseDisplay.userGuid (current userguid)
		// caseDisplay.caseinfo.workItemId (workitemId to display if any - taken from URL/route)

		//common functions

		displayCaseDetails = function (caseReference) {
			//console.log("*&* CaseDisplay Controller: displayCaseDetails for caseRef : " + caseReference);
			//vm.caseinfo = {};

			vm.caseinfo.caseRef = caseReference;
			vm.caseinfo.caseReference = caseReference;

			//Use rootScope to set last visited case
			$rootScope.lastCaseRef = caseReference;

			CaseInformationService.getCaseData(vm.caseinfo, vm.caseinfo.caseRef, {
                useDefault: false,
				onSuccess: function () {
					vm.caseinfo.casedata = {};
					var tmpCaseData = vm.caseinfo.caseDataTree;
					for (var i = 0; i < tmpCaseData.length; i++) {
						var dataField = tmpCaseData[i];
						if (dataField.value != null) {
							vm.caseinfo.casedata[dataField.name] = dataField.value;
						}
					}

					// parse the case summary to create a more readable structure and store in caseDataAll object
					var caseData = tmpCaseData;
					// iterate through the list to create a more readable collection
						parseCaseTree = function(caseDataTree, parentTree, level, target, parent) {
							for (var i = 0; i < caseDataTree.length; i++) {
								var attribute = caseDataTree[i];

								if (!attribute.children) {
									//console.log('this is an attribute: ' + attribute.name + ' level ' + level);
									target[attribute.name] = attribute.value;
								} else {
									// This is a child object so put it in a child object then make it an array, then create child elements in that array
									//console.log('this is an object: ' + attribute.name + ' level ' + level);
									// check if it already exists
									if (target[attribute.name]== undefined) {
										target[attribute.name] = [];
									}
									var len = target[attribute.name].length;
									target[attribute.name][len] = {};
									parseCaseTree(attribute.children, caseDataTree, level + 1, target[attribute.name][len]);
								}
							}
						}
						vm.caseinfo.caseDataAll = {};
						parseCaseTree(caseData, null, 0, vm.caseinfo.caseDataAll);

					vm.spinner.stop();
					vm.dataloaded = true;

					$rootScope.$digest();
				}, onFailure: function (error) {
                    vm.spinner.stop();
                    console.error("ERR_CASE_DISPLAY_CTLR_GET_CASE_DATA: " + error);
				}

			});
		}

		getSummary = function (caseReference) {
			vm.dataloaded = false;

			var opts = {
				lines: 13 // The number of lines to draw
				, length: 28 // The length of each line
				, width: 12 // The line thickness
				, radius: 42 // The radius of the inner circle
				, scale: 0.75 // Scales overall size of the spinner
				, corners: 0.6 // Corner roundness (0..1)
				, color: '#000' // #rgb or #rrggbb or array of colors
				, opacity: 0.25 // Opacity of the lines
				, rotate: 0 // The rotation offset
				, direction: 1 // 1: clockwise, -1: counterclockwise
				, speed: 1.2 // Rounds per second
				, trail: 86 // Afterglow percentage
				, fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
				, zIndex: 2e9 // The z-index (defaults to 2000000000)
				, className: 'spinner' // The CSS class to assign to the spinner
				, top: '50%' // Top position relative to parent
				, left: '50%' // Left position relative to parent
				, shadow: false // Whether to render a shadow
				, hwaccel: false // Whether to use hardware acceleration
				, position: 'absolute' // Element positioning
			}
			var target = document.getElementById('loadingSpinner');
			vm.spinner = new Spinner(opts).spin(target);
			vm.summarydata = {};
			vm.summary = {};
			vm.summary.caseRef = caseReference;
			// Assign case class details
			caseRefParsed = vm.summary.caseRef.split("-");
			vm.caseinfo.caseClass = caseRefParsed[2];
			// Get the short class name - caseType
			caseClassParsed = vm.caseinfo.caseClass.split(".");
			vm.caseinfo.caseType = caseClassParsed[caseClassParsed.length - 1];
			CaseInformationService.getCaseInformation(vm.summary, {
                useDefault: false,
				onSuccess: function () {
					//Service call assigns the case summary	to scope.caseRefInformation
					// also save the current state and caseId to vm.caseinfo
					for (var i = 0; i < vm.summary.caseRefInformation[0].details.length; i++) {
						if (vm.summary.caseRefInformation[0].details[i].isCaseIdentifier == 'true') {
							vm.caseinfo.caseId = vm.summary.caseRefInformation[0].details[i].value;
							vm.caseinfo.caseIdFieldLabel = vm.summary.caseRefInformation[0].details[i].label;
							vm.caseinfo.caseIdFieldName = vm.summary.caseRefInformation[0].details[i].name;
						}
						if (vm.summary.caseRefInformation[0].details[i].isState == 'true') {
							vm.caseinfo.state = vm.summary.caseRefInformation[0].details[i].value;
							vm.caseinfo.stateFieldLabel = vm.summary.caseRefInformation[0].details[i].label;
							vm.caseinfo.stateFieldName = vm.summary.caseRefInformation[0].details[i].name;
						}
						thisField = vm.summary.caseRefInformation[0].details[i].name;
						vm.summarydata[thisField] = {};
						vm.summarydata[thisField].label = vm.summary.caseRefInformation[0].details[i].label;
						vm.summarydata[thisField].value = vm.summary.caseRefInformation[0].details[i].value;
					}
					vm.summaryRead = true;
					// Save the states for this case
					// loop to find the case model
					for (var i = 0; i < vm.casemodels.length; i++) {
						if (vm.casemodels[i].name === vm.caseinfo.caseClass) {
							vm.caseinfo.casemodel = vm.casemodels[i].attributeInfo;
						}
					}
					// loop to find possible case states
					vm.caseinfo.caseAttributeLabels = {};
					for (var i = 0; i < vm.caseinfo.casemodel.length; i++) {
						// save this label so we can use it later when calling readCase
						vm.caseinfo.caseAttributeLabels[vm.caseinfo.casemodel[i].path] = vm.caseinfo.casemodel[i].label;
						if (vm.caseinfo.casemodel[i].path === vm.caseinfo.stateFieldName) {
							vm.caseinfo.states = vm.caseinfo.casemodel[i].enumLiteral;
						}
					}
					displayCaseDetails(vm.caseRef);
				}
                , onFailure: function (error) {
                    vm.spinner.stop();
                    alert("Unable to find this case - does this case still exist?")
                    console.error("ERR_CASE_DISPLAY_CTLR_GET_CASE_INFORMATION: " + error);
                }
			});
		}

		getCaseModels = function () {
			CaseDisplayBPMServices.getCaseModel().then(
					function (result) {
						vm.casemodels = result.caseClassInfo;
						getSummary(vm.caseRef);
						vm.getLinkedCases();
					},
					function (error) {
                        console.error("ERR_CASE_DISPLAY_CTLR_GET_CASE_MODEL: " + error);
					}
			);
		}


		// Start of controller execution

		var vm = this;

		vm.dataloaded = false;
		vm.caseinfo = {};
		vm.casemodels = {};
		vm.summarydata = {};
		vm.casedata = {};
		vm.caseinfo.caseDataTree = {};
		vm.summaryRead = false;

		vm.topClass = true;
		vm.topClassM = true;
		vm.topClassT = true;
		vm.taskTabs = true;

		vm.pos = {}
		vm.currentPos = 10;


		vm.bizActionFormTabs = [];

		vm.$mdMedia = $mdMedia;


		checkFavourite = function () {
			CaseDisplayBPMServices.readCaseFavourites($rootScope.userResourceDetails.guid).then(
					function (result) {
						var settingProperties = result['settingProperties'];
						if (settingProperties == undefined) {
							vm.caseinfo.favourite = false;
						} else {
                            if (settingProperties['@Value'] != undefined) {
                                var favouriteCaseRefs = settingProperties['@Value'].split(',');
							} else {
                                var favouriteCaseRefs = settingProperties[0]['@Value'].split(',');
							}
							var shortCaseRef = vm.caseinfo.caseRef.slice(0,vm.caseinfo.caseRef.lastIndexOf("-"));
							var shortCaseRef = shortCaseRef + '-0';
							if (favouriteCaseRefs.indexOf(shortCaseRef) != -1) {
								vm.caseinfo.favourite = true;
							} else {
								vm.caseinfo.favourite = false;
							}
						}
					},
					function (error) {
						console.error("ERR_CASE_DISPLAY_CTLR_READ_CASE_FAVOURITES: " + error);
					}
            )
		}

		checkFavourite();

		vm.setHeaderTemplate = function () {
			vm.customHeaderTemplate = true;
		}

		vm.setBodyTemplate = function () {
			vm.customBodyTemplate = true;
		}

		vm.toggle = function (name) {
			if (vm[name] == undefined) {
				vm[name] = true;
			} else {
				vm[name] = !vm[name];
			}
			if ((vm[name] == true) || (name == "overviewShow")) {
				vm.currentPos--
				vm.pos[name + 'Pos'] = vm.currentPos;
				$anchorScroll();
			}
		}

		vm.refresh = function () {
			getSummary(vm.caseinfo.caseRef);
		}

		$scope.navigateToCase = function (data) {
			$scope.caseRef = data;
			vm.caseRef = data;
			vm.caseinfo.caseRef = data;
			vm.caseinfo.workItemId = undefined;
			vm.refresh();
		}

		//Get caseRef and workItemId, then get case data tree
		vm.caseRef = $scope.caseRef;
		vm.caseinfo.caseRef = $scope.caseRef;
		vm.caseinfo.workItemId = $scope.workItemId;

		BPMLoginService.getExtendedUserInfo({
            useDefault: false,
			onSuccess: function (extUserInfo) {
				vm.userGuid = extUserInfo.guid;
				vm.userName = extUserInfo.name;
				if (vm.caseRef) {
					//CaseDisplayBPMServices.recordCaseAccess(vm.userGuid, vm.caseRef);
					CaseDisplayBPMServices.logCaseView(vm.caseRef, vm.userGuid, vm.userName).then(
							function (result) {

							},
							function (error) {
								console.error("ERR_CASE_DISPLAY_CTLR_LOG_CASE_VIEW: " + error);
							}
					);

					// Get Case Model details
					getCaseModels();
				}
			},
            onFailure: function (error) {
            	console.error("ERR_CASE_DISPLAY_CTLR_GET_EXT_USER_INFO: " + error);
        	}
		});
		
				// get linked cases
		vm.getLinkedCases = function() {
			CaseDisplayBPMServices.getNavigateDetailed(vm.caseRef).then(
				function(result){
					vm.caseinfo.linkedCaseDetailsAPI = result.caseObjectsByRole;
					var linkedCaseDetails = result.caseObjectsByRole;
					vm.caseinfo.linkedCaseDetails = {};

					// iterate through the list to create a more readable collection
					for (var i = 0; i < linkedCaseDetails.length; i++) {
						vm.caseinfo.linkedCaseDetails[linkedCaseDetails[i].roleName] = [];
						angular.forEach(linkedCaseDetails[i].caseObjectDetails, function (linkedCase) {
							var casedef = {};
							casedef.attributes = [];
							casedef.caseRef = linkedCase.caseReference;
							angular.forEach(linkedCase.details, function(caseDetails) {
								if (caseDetails.isCaseIdentifier) {
									casedef.caseId = caseDetails.value;
								}
								casedef.attributes[caseDetails.name] = caseDetails.value;
							})
							vm.caseinfo.linkedCaseDetails[linkedCaseDetails[i].roleName].push(casedef);
						})
					}


				},
                function (error) {
                    console.error("ERR_CASE_DISPLAY_CTLR_GET_LINKED_CASES: " + error);
                }
			)
		};


	vm.toggleFavourite = function(value) {
		if (value) {
			// was set to true
			CaseDisplayBPMServices.addToCaseFavourites($rootScope.userResourceDetails.guid, vm.caseRef);
		} else {
			CaseDisplayBPMServices.removeFromCaseFavourites($rootScope.userResourceDetails.guid, vm.caseRef);
			// was set to false
		}
	}

	vm.actionOpenWi = function(wiId) {
		if (vm.caseinfo.workItemId === undefined) {
			vm.caseinfo.workItemId = wiId;
		} else {
			alert('Please complete the currently displayed workitem first');
		}
	}
		
	vm.runActionInNewTab = function (businessActionTemplate) {

            businessActionTemplate.formDiv = "formBizDiv#" + businessActionTemplate.itemData.processName;
            businessActionTemplate.tabName = businessActionTemplate.itemData.caseActionName;

            for(i=0;i<vm.bizActionFormTabs.length;i++)
            {
                if((vm.bizActionFormTabs[i].itemData != null) && (vm.bizActionFormTabs[i].itemData.processName==businessActionTemplate.itemData.processName))
                    return;
            }

            vm.bizActionFormTabs.push(businessActionTemplate);
			CaseDisplayBPMServices.logCaseAudit(vm.caseRef, vm.userName, vm.userGuid, businessActionTemplate.itemData.processName, vm.caseinfo.state, 'CMF_CASE_ACTION_STARTED', 'Case action started');
			
            BPMBusinessActionsService.startBizAction(businessActionTemplate, {
                useDefault: false,
            	onSuccess: function (result) {
					// {\"items\":[{\"$param\":\"caseRef\",\"$value\":[\"BDS-1-Service.Cases.Customer-4165-0\"],\"type\":\"String\",\"mode\":\"OUT\"}]}"}}]}"
					// if we got a parameter back - check if it is caseRef. If it is navigate to the case page
					if (result.pageData != undefined) {
                        if (result.pageData[0] != null && result.pageData[0].payload != null && result.pageData[0].payload.serializedPayload != null) {
                            var resultPayload = result.pageData[0].payload.serializedPayload;
                            resultParams = JSON.parse(resultPayload);
                            var caseRef;
                            angular.forEach(resultParams.items, function (item) {
                                if (item.$param == 'caseRef') {
                                    // found the caseRef
                                    caseRef = item.$value[0];
                                }
                            })
                            if (caseRef != null) {
                                // navigate to case
                                ////console.log("Setting caseref to: " + $scope.marking.case_ref);
                                $location.path("/pageCaseDisplay/" + caseRef);
                                $scope.$apply();
                            } else {
                                vm.refresh();
                            }
                        } else {
                            vm.refresh();
                        }
                    }
                },
                onFailure: function (error) {
                    console.error("ERR_CASE_DISPLAY_CTLR_START_BIZ_ACTION: " + error);
                    $timeout(function() {
                        vm.bizActionFormTabs.splice(vm.bizActionFormTabs.indexOf(businessActionTemplate), 1);
                        vm.$apply("");
                    });
                },
				onEvent: function(eventName, bizActionHandler){
					if (eventName == 'bs.COMPLETE' || eventName == 'bs.CANCEL') {
							vm.bizActionFormTabs.splice(vm.bizActionFormTabs.indexOf(businessActionTemplate), 1);
							if (eventName === 'bs.COMPLETE') {
								// TODO: Jez - what if we have two actions?
								// should we show the the current state here or the state when the action was started? For now it shows the state when the action was started - probably better for correlation of audit messages
								CaseDisplayBPMServices.logCaseAudit(vm.caseRef, vm.userName, vm.userGuid, businessActionTemplate.itemData.processName, vm.caseinfo.state, 'CMF_CASE_ACTION_COMPLETED', 'Case action completed');
								vm.bizActionFormTabs = [];
							} else {
								// otherwise its a cancel
								$scope.$apply();
							}
					}
				}
            });
        }
	}


	
]);