/**
 * Created by JeremyS on 3/1/17.
 */

angular.module('components.casePendingList', ['components.payloadService'])
    .controller('CasePendingListController', ['$scope', '$rootScope', 'CaseDisplayBPMServices', 'BPMWorklistService', 'CaseInformationService', 'BPMBusinessActionsService', 'PayloadService', '$mdToast', function ($scope, $rootScope, CaseDisplayBPMServices, BPMWorklistService, CaseInformationService, BPMBusinessActionsService, PayloadService, $mdToast) {
        //console.log("*** Case Pending List Controller");
		var vm = this;
        $scope.displayFullPane = false;

        // only show the full pane if we have workitems/activities to process
        //$scope.displayFullPane = false;

        // used so we don't display the activity info until all the service calls are complete
        $scope.display=false;
        $scope.displayActions=false;
        $scope.displayAdhoc=false;

        // used to show counters for each activity type
        $scope.activityCounters = {};
        $scope.activityCounters.activities = 0;
        $scope.activityCounters.adhoc = {};
        $scope.activityCounters.adhoc.addable = 0;
        $scope.activityCounters.adhoc.running = 0;
        $scope.activityCounters.actions = 0;

        // we re-organize the adhoc tasks list so its easy to display them on a template
        $scope.activities = {};
        $scope.activities.addableActivities = [];
        $scope.activities.runningActivities = [];

        $scope.setDisplayFullPane = function() {
            $scope.displayFullPane = true;
        }

        $scope.unsetDisplayFullPane = function() {
            $scope.displayFullPane = false;
        }

        // show a toast popup if there are tasks pending
        $scope.showCustomToast = function(tasksPending) {
            $mdToast.show({
                hideDelay   : 60000,
                position    : 'top right',
                controller  : 'CasePendingToastCtrl',
                templateUrl : 'components/case-pending-list/templates/case-pending-toast.html',
                locals:{activities:tasksPending, displayFullPane:$scope.setDisplayFullPane}
            });
        };

        findWiInWiList = function(wiId, wiList){
            // this function checks whether a workitem with matching wiId is in the supplied wiList of workitems
            var found = false;
            angular.forEach(wiList, function (wi) {
                if (wi.id.id === wiId ) {
                    found = true;
                }
            });
            return found;
        };

        $scope.openWi = function(wiId) {
            $scope.actionOpenWi({wiId: wiId});
        }

        // retrieve all outstanding pending activities for this case (workitems for the case reference)
        $scope.getPendingActivities = function(caseRef) {

            // First get the worklist for this case
            var startPosition = '0';
            var numberOfItems = '100';
            var globalDataRef = caseRef;
            var model ={};
            model.templateData = {};
            BPMWorklistService.getWorkListItemsForGlobalData(startPosition, numberOfItems, globalDataRef, model, {
                onSuccess: function (response) {
                    $scope.caseWl = response.templateData;

                        // now get the user's worklist
                        var worklistServiceModel = {};
                        worklistServiceModel.templateData = {};
                        var start = '0';
                        var count = '100';
                        var entityType = 'RESOURCE';
                        var entityGuid = $rootScope.userResourceDetails.guid;

                        BPMWorklistService.getWorklistItems(entityType,entityGuid,start,count, worklistServiceModel, {
                            onSuccess: function (response) {
                                $scope.userWl = response.templateData;

                                // now highlight items not in the user's worklist because they cant perform those
                                // ...
                                angular.forEach($scope.caseWl.workItems, function (caseWi) {
                                    if (findWiInWiList(caseWi.id.id, $scope.userWl.workItems)) {
                                        caseWi.available = true;
                                    } else {
                                        caseWi.available = false;
                                    }

                                    // Get offerset/resources who can process the item
                                    // will be processed async
                                    CaseDisplayBPMServices.getOfferSet(caseWi.id.id).then(
                                        function (result) {
                                            caseWi.offerset = {};
                                            caseWi.offerset = result;
                                            caseWi.offerset.resources = [];
                                            var offerGuid = [];
                                            if (caseWi.offerset.entityGuid == undefined) {
                                                // item was offered using data field or RQL performer
                                                // get resources for each entity guid
                                                angular.forEach(caseWi.offerset.entityId, function (entity) {

                                                    // get entity name
                                                    CaseDisplayBPMServices.getOrgModelEntity(entity.modelVersion, [entity.guid]).then(
                                                        function (result) {
                                                            entity.entityDetails = result;
                                                        }
                                                    )
                                                    // add guid to caseWi object
                                                    offerGuid.push(caseWi.offerset.entityId[0].guid);
                                                });
                                            } else {
                                                // WI was offered using entity (legacy support?)
                                                offerGuid.push(caseWi.offerset.entityGuid);
                                            }
                                                for (n=0; n<offerGuid.length;n++) {
                                                    if (offerGuid[n] != 'Undelivered') {
                                                        CaseDisplayBPMServices.getResourcesForEntity(offerGuid[n]).then(
                                                            function (result) {
                                                                caseWi.offerset.resources.push(result);
                                                            }
                                                        )
                                                    }
                                                }
                                        }
                                    );
                                });
                                if ($scope.caseWl.workItems != undefined) {
                                    $scope.activityCounters.activities = $scope.caseWl.workItems.length;
                                } else {
                                    $scope.activityCounters.activities = 0;
                                }

                                $scope.display = true;
                                // show the full pane if we have workitems/activities to process
                                /*if ($scope.activityCounters.activities > 0 ) { $scope.displayFullPane = true };
                                $scope.$apply();*/

                                // now we can list $scope.caseWi and highlight the items a user can perform (see in their worklist)
                            }
                        });
                }
            });

        }

        // get the action list for this caseref
        $scope.getAvailableActions = function(caseRef) {
            // get the available runable actions
            $scope.caseRef = caseRef;
            CaseInformationService.listCaseActions($scope,$scope.caseRef, {
                onSuccess: function()
                    {
                        if ($scope.caseActions != undefined) {
                            $scope.activityCounters.actions = $scope.caseActions.length;
                            // only show actions when someone clicks on the actions header
                            //$scope.displayActions = true;
                        }
                    }
            });
        }

        // get and re-organize the adhoc tasks list for this caseref
        $scope.getAdhocActivities = function(caseRef) {
            // get adhoc activities
            CaseDisplayBPMServices.listAdhocActivitiesByGlobalRef(caseRef).then(
                function (result) {
                    $scope.adHocTasks = result.activity;
                    if ($scope.adHocTasks != undefined) {
                        for (var i = 0; i < $scope.adHocTasks.length; i++) {
                            if ($scope.adHocTasks[i].enabled && !$scope.adHocTasks.automatic) {
                                if (!$scope.adHocTasks[i].inProgress || ($scope.adHocTasks[i].inProgress && $scope.adHocTasks[i].repeatable)) {
                                    $scope.activities.addableActivities.push($scope.adHocTasks[i]);
                                    $scope.activityCounters.adhoc.addable++;
                                }
                            }
                            if ($scope.adHocTasks[i].inProgress) {
                                $scope.activities.runningActivities.push($scope.adHocTasks[i]);
                                $scope.activityCounters.adhoc.running++;
                            }
                        }
                    }
                    // show the full pane if we have addable or running adhoc activities to process
                    /*if ($scope.activityCounters.adhoc.addable > 0 || $scope.activityCounters.adhoc.running > 0) { $scope.displayFullPane = true };*/
                    $scope.displayAdhoc = true;
                });
        }

        // run an adhoc activity
        $scope.runAdhocActivity = function(adhocActivity) {
            CaseDisplayBPMServices.runAdhocActivity(adhocActivity.processID, adhocActivity.name).then(
                function (result) {
                    CaseDisplayBPMServices.logCaseAudit($scope.caseReference, $rootScope.userResourceDetails.name, $rootScope.userResourceDetails.guid, adhocActivity.name, $scope.caseState, 'CMF_CASE_ADHOC_TASK_STARTED', 'Adhoc task started');
                    alert('Added activity ' + adhocActivity.name);
                    $scope.triggerRefresh();
                });
        }

        // cancel an adhoc activity
        $scope.cancelAdhocActivity = function(adhocActivity) {
            CaseDisplayBPMServices.cancelAdhocActivity(adhocActivity.processID, adhocActivity.name).then(
                function (result) {
                    CaseDisplayBPMServices.logCaseAudit($scope.caseReference, $rootScope.userResourceDetails.name, $rootScope.userResourceDetails.guid, adhocActivity.name, $scope.caseState, 'CMF_CASE_ADHOC_TASK_CANCELLED', 'Adhoc task cancelled');
                    alert('Cancelled activity ' + adhocActivity.name);
                    $scope.triggerRefresh();
                });
        }

        // run an action
        $scope.runAction = function(action) {
            //run the bizAction thanks to a passed method in the directive
                //console.log("********* Do Action with action : " + JSON.stringify(action));
                var caseTemplate = action.caseActionTemplate[0];

                //console.log("********* Do Action with template : " + JSON.stringify(caseTemplate));

                var businessActionTemplate = {};
                businessActionTemplate.itemData = caseTemplate;
                businessActionTemplate.payload = PayloadService.createCaseActionPayload($scope.caseReference, caseTemplate.caseRefParamName);
                //console.log("*******************Case Action payload : " + businessActionTemplate.payload);
                businessActionTemplate.formDiv = "workItem";

                $scope.actionMethod({actionTemplate: businessActionTemplate});
        }

        $scope.setFullDisplayPane = function() {
            $scope.displayFullPane = true;
        }

        $scope.disableFullDisplayPane = function() {
            $scope.displayFullPane = false;
        }

        $scope.toggleActionDisplay = function() {
            $scope.displayActions = !$scope.displayActions;
        }

        $scope.getResourceDetails = function(wi) {
            //console.log('called with: ' + JSON.stringify(wi.offerset.resources.length));
            var resources = [];
            for (n=0;n<wi.offerset.resources.length; n++) {
                //console.log('1');
                var resourceList = wi.offerset.resources[n];
                //console.log('2');
                for (i = 0; i < resourceList.resource.length; i++) {
                    //console.log('3 ', resourceList.resource[i].label);
                    resources.push(resourceList.resource[i].label);
                    //console.log('added');
                }
            }
            console.log("resources: " + resources);
            if (resources != '') {
                if (resources.size <= 5) {
                    alert('The following users match this task?\n\n' + resources);
                } else {
                    alert('There are ' + resource.size + ' resources that can process this task');
                }
            } else {
                alert('There are currently no resources who match this task\n\n' + resources);
            }
        }

        // if there are activities for this case then display a notification
        $scope.$watch('activityCounters.activities', function(newVal) {
            if (newVal > 0 && $rootScope.casePendingList.notificationsEnabled && !$scope.displayFullPane) {
                $scope.showCustomToast(newVal);
            }
        })

        $scope.closeNotification = function() {
            $mdToast.hide();
        }

        $scope.$on('$destroy', function() {
            $scope.closeNotification();
        });
    }])

    .controller('CasePendingToastCtrl', ['$scope', '$mdToast', 'activities', 'displayFullPane', function($scope, $mdToast, activities, displayFullPane) {
        $scope.activities = activities;
        $scope.displayFullPane = displayFullPane;
        $scope.closeToast = function() {
            $mdToast
                .hide()
        };

        $scope.openMoreInfo = function(e) {
            $scope.displayFullPane();
            $mdToast
                .hide()
        };
    }]);
