/**
 * Created by JSmith on 20/09/2017.
 */

angular.module('components.genericWebplayer', [])
    .controller('GenericWebplayerController', ['$scope', function ($scope, $timeout, $mdDialog, $rootScope) {

        /********************/
        /* functions        */
        /********************/

        var openedCallback = function (analysisDocument) {

            // If required you can define a filter scheme in the vizualization, then pass the filter when opening webplayer.
            // You can also pass filterValues - example filter viz for customerId X
            // if you specify a filter - we apply the filter to the viz when opened
            // note make sure the filterScheme you select is the current filter scheme in the visualization
            if ($scope.filterScheme != undefined && $scope.filterScheme != "" ) {
                var filterColumn = {
                    filteringSchemeName: $scope.filterScheme,
                    dataTableName: $scope.filterTable,
                    dataColumnName: $scope.filterColumn,
                    filteringOperation: spotfire.webPlayer.filteringOperation.REPLACE,
                    filterSettings: {
                        includeEmpty: false,
                        values: $scope.filterValues
                    }
                };

                var filteringOperation = spotfire.webPlayer.filteringOperation.REPLACE;

                analysisDocument.filtering.setFilter(
                    filterColumn,
                    filteringOperation);
            }

            // this is for when we are looking at the BDS tables and want to create a caseref in markings
            // push on the standard case related columnNames
            if ($scope.bdsVersion != -1) {
                var bdsColumns = ['BDS_ID', 'DTYPE', 'BDS_VERSION'];
                $scope.columnNames = $scope.columnNames.concat(bdsColumns);
            }

            // register the marking callback function
            analysisDocument.marking.onChanged(
                $scope.markingName,
                $scope.tableName,
                $scope.columnNames,
                $scope.maxMarkings,
                onMarkingChangedCallback);

            // all done so show the vizualization
            $scope.sfLoaded = true;
        };

        var onMarkingChangedCallback = function (marking) {
            var col1 = $scope.columnNames[0];
            var selection_size = 0;
            var selection = [];

            if (marking[col1] != undefined) {
                selection_size = marking[col1].length;
                if (selection_size <= $scope.maxMarkings) {
                    for (var i = 0; i < selection_size; i++) {
                        var elt = {};
                        $scope.columnNames.forEach(
                            function (s) {
                                elt[s] = marking[s][i];
                            });
                        // if you are looking at a BDS table we can calculate the case reference
                        // format: BDS-1-Service.Cases.Complaint-5-1
                        if ($scope.bdsVersion != -1) {
                            elt['case_ref'] = 'BDS-' + $scope.bdsVersion + '-' + marking['DTYPE'][i] + '-' + marking['BDS_ID'][i] + '-0';
                        }
                        selection.push(elt);
                    }
                } else {
                    // gone over max marking count so ignore it
                    selection_size = "max";
                }
            }
            // now call the callback we were passed (passed as directive parameter)
            $scope.callback({
                selection_size: selection_size,
                selection: selection
            });

        };

        var errorCallback = function (errorCode, description) {
            // Displays an error message if something goes wrong in the Web Player.
            console.warn("Spotfire WP exception: " + errorCode + ": " + description);
        }

// launch the web player
        var launchPlayer = function (analysisPath, divId, showToolBar) {
            var spotserver = $scope.spotfireServer;
            var webPlayerCustomization = new spotfire.webPlayer.Customization();
            webPlayerCustomization.showCustomizableHeader = false;
            webPlayerCustomization.showTopHeader = false;
            webPlayerCustomization.showClose = false;
            webPlayerCustomization.showAnalysisInfo = false;
            webPlayerCustomization.showToolBar = showToolBar;
            webPlayerCustomization.showExportFile = false;
            webPlayerCustomization.showExportVisualization = false;
            webPlayerCustomization.showUndoRedo = false;
            webPlayerCustomization.showDodPanel = false;
            webPlayerCustomization.showFilterPanel = $scope.showFilterPanel;
            webPlayerCustomization.showStatusBar = false;
            webPlayerCustomization.showPageNavigation = $scope.showNavigatation;


            webPlayer = new spotfire.webPlayer.Application(spotserver, webPlayerCustomization);
            // Register an error handler to catch errors.
            webPlayer.onError(errorCallback);
            var params = '';
            // Open an analysis
            // path , div in the dom to inject the webplayer, params
            // now load the web player into your web page
            webPlayer.onOpened(openedCallback);
            webPlayer.open(
                analysisPath, // the path to your project in the Spotfire library
                divId, // this is the div tag in your HTML page you want to the web player to appear inside
                params // this last option is for if you need to pass your web player parameters for a parameterised information link
            );
        };


        /********************/
        /* controller logic */
        /********************/

        console.log("*** GenericWebplayerController");

        // launch the webplayer
        var webPlayer;
        var result = launchPlayer($scope.analysisPath, $scope.webplayerDiv, $scope.showToolBar);

    }]);
