/**
 * Created by Jez on 6/9/2016.
 */

angular.module('components.recentCasesDisplay', [])
    .controller('RecentCasesDisplayController', ['$scope', '$rootScope', "CaseDisplayBPMServices", "CaseManagementService", "CaseInformationService", "$location", function ($scope, $rootScope, CaseDisplayBPMServices, CaseManagementService, CaseInformationService, $location) {
		//console.log("*****************RecentCasesDisplay Controller");

		$scope.ready = false;

		$scope.openCase = function (caseRef) {
			$location.path("/pageCaseDisplay/"+caseRef);
		}

		// use EC to get recently touched cases (we limit amount returned from API to the 50 most recent for performance reasons)
		CaseDisplayBPMServices.retrieveCaseAccessEC(50).then(
			function(result) {
				$scope.recentCasesEC = [];
				// first make the list a more readable structure
				if (result.totalItems > 0) {
					for (x = 0; x < result.resultEntry.length; x++) {
						var singleResult = result.resultEntry[x];
						var recentCase = {};
						// create a clean case entry rather than name/value pair
						angular.forEach(singleResult.attribute, function (attribute) {
							// if managedObjectId (caseRef) replace the trailing version with -0
							if (attribute.attributeName === 'managedObjectId') {
								var caseRef = attribute.value;
								var genericCaseRef = caseRef.slice(0, caseRef.lastIndexOf("-"));
								var genericCaseRef = genericCaseRef + '-0';
								attribute.value = genericCaseRef;
								recentCase.caseRef = attribute.value;
							} else if (attribute.attributeName === 'creationTime') {
								recentCase.lastViewed = attribute.value;
							} else {
								recentCase[attribute.attributeName] = attribute.value;
							}

							// extract the case type (short class name) from caseRef
							var caseRefString = recentCase.caseRef;
							var seploc = caseRefString.indexOf('-');
							var BDSHeader = caseRefString.slice(0, seploc);
							caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
							var seploc = caseRefString.indexOf('-');
							var MajorVers = caseRefString.slice(0, seploc);
							caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
							seploc = caseRefString.indexOf('-');
							var ClassName = caseRefString.slice(0, seploc);
							caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
							seploc = caseRefString.indexOf('-');
							var shortClassName = ClassName.slice(ClassName.lastIndexOf(".") + 1);
							var BDSId = caseRefString.slice(0, seploc);
							recentCase.class = ClassName;
							recentCase.type = shortClassName;


						});
						$scope.recentCasesEC.push(recentCase);
					}

					// now remove duplicates (those with same caseId
					var seen = {};
					var uniqueCaseAccessRecord = $scope.recentCasesEC.filter(function (item) {
						if (seen.hasOwnProperty(item.caseRef)) {
							return false;
						} else {
							seen[item.caseRef] = true;
							return true;
						}
					});
					$scope.recentCasesEC = uniqueCaseAccessRecord;

					// now get more info about each case
					angular.forEach($scope.recentCasesEC, function (recentCase) {
						// get case summary (async)
						var summary = {};
						summary.caseRef = recentCase.caseRef;
						// useDefault: false on callback to getCaseInformation supresses error messages when case has been deleted
						CaseInformationService.getCaseInformation(summary,
								{
									useDefault: false,
									onSuccess: function () {
										// parse the case summary to create a more readable structure and store in recentCase object
										recentCase.fieldSummary = {};
										angular.forEach(summary.caseRefInformation[0].details, function (caseAttribute) {
											if (caseAttribute.isState) {
												recentCase.fieldSummary.STATE = {}
												recentCase.fieldSummary.STATE.label = caseAttribute.label;
												recentCase.fieldSummary.STATE.value = caseAttribute.value;

												// set friendly state labels
												var statesModel = {};
												statesModel.caseReference = summary.caseRef;
												statesModel.caseRefArray = [];
												statesModel.caseRefArray.push(summary.caseRef);
												CaseManagementService.getCaseStates(statesModel,{
													useDefault: false,
													onSuccess: function()
														{
															recentCase.fieldSummary.STATE.valueLabel = statesModel.caseStates[0].value.label;

														},
													onFailure: function(error) {
															// handle if case wasnt there any more
															recentCase.fieldSummary.STATE.valueLabel = 'Case Deleted';
													}
												});
											}
											if (caseAttribute.isCaseIdentifier) {
												recentCase.fieldSummary.CASEID = {}
												recentCase.fieldSummary.CASEID.label = caseAttribute.label;
												recentCase.fieldSummary.CASEID.value = caseAttribute.value;
											}
											recentCase.fieldSummary[caseAttribute.name] = {}
											recentCase.fieldSummary[caseAttribute.name].label = caseAttribute.label;
											recentCase.fieldSummary[caseAttribute.name].value = caseAttribute.value;
										});
										recentCase.deleted = false;
										$scope.$apply();
										$scope.ready = true;
									},
									onFailure: function(error) {
										console.info('Unable to retrieve case information, case has probably been deleted \n' + JSON.stringify(error));
										recentCase.deleted = true;
										$scope.$apply();
									}
								});
					});
				}
			});
	}]);

		// no longer use user preferences for this
/*
		CaseDisplayBPMServices.retrieveCaseAccess($rootScope.userResourceDetails.guid, 5).then(
				function(result) {
					$scope.recentCases = result;

					angular.forEach($scope.recentCases, function (recentCase) {
						// get case summary (async)
						recentCase.summary = {};
						recentCase.summary.caseRef = recentCase.caseRef;
						CaseInformationService.getCaseInformation(recentCase.summary,
								{
									onSuccess: function () {
										// parse the case summary to create a more readable structure and store in recentCase object
										recentCase.fieldSummary = {};

										// extract type from caseRef
										var caseRefString = recentCase.caseRef;
										var seploc = caseRefString.indexOf('-');
										var BDSHeader = caseRefString.slice(0,seploc);
										caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
										var seploc = caseRefString.indexOf('-');
										var MajorVers = caseRefString.slice(0,seploc);
										caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
										seploc = caseRefString.indexOf('-');
										var ClassName = caseRefString.slice(0,seploc);
										caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
										seploc = caseRefString.indexOf('-');
										var shortClassName = ClassName.slice(ClassName.lastIndexOf(".")+1);
										var BDSId = caseRefString.slice(0,seploc);
										recentCase.class = ClassName;
										recentCase.type = shortClassName;

										angular.forEach(recentCase.summary.caseRefInformation[0].details, function (caseAttribute) {
											if (caseAttribute.isState) {
												recentCase.fieldSummary.STATE = {}
												recentCase.fieldSummary.STATE.label = caseAttribute.label;
												recentCase.fieldSummary.STATE.value = caseAttribute.value;
											}
											if (caseAttribute.isCaseIdentifier) {
												recentCase.fieldSummary.CASEID = {}
												recentCase.fieldSummary.CASEID.label = caseAttribute.label;
												recentCase.fieldSummary.CASEID.value = caseAttribute.value;
											}
											recentCase.fieldSummary[caseAttribute.name] = {}
											recentCase.fieldSummary[caseAttribute.name].label = caseAttribute.label;
											recentCase.fieldSummary[caseAttribute.name].value = caseAttribute.value;
										});
										$scope.$apply();
									}
								});
					});
					$scope.ready = true;
				}
		);
	*/

