/**
 * Created by Jez on 14/6/2016.
 */

angular.module('components.searchBoxDisplay', ['ui.bootstrap', 'angular-advanced-searchbox', 'components.SrcDisplayServices'])
    .controller('SearchBoxDisplayController', ['$scope', "$timeout", "SrcAppBPMServices", function ($scope, $timeout, SrcAppBPMServices) {
		//console.log("*****************SearchBoxDisplayDirective Controller", $scope.selectedClass, $scope.searchParams, $scope.maxResults);

		$scope.loaded = false;


		// utility function to check if object is empty
		function isEmpty(object) {
			for (var key in object) {
				if (object.hasOwnProperty(key)) {
					return false;
				}
			}
			return true;
		}

		// create search params from case model
		buildSearchParams = function (model) {
			$scope.availableSearchParams = [];

			for (var i = 0; i < model.length; i++) {
				// disabled DATETIME for now
				//TODO implement DATETIME support for search
				if (model[i].type != 'ATTR_DATETIME') {
					var param = {};
					param.key = model[i].path;
					//param.name = model[i].path;
					param.name = model[i].label;
					param.type = model[i].type;
					param.label = model[i].label;
					param.placeholder = model[i].label + '...';
					if (model[i].enumLiteral != null) {
						param.restrictToSuggestedValues = true;
						param.suggestedValues = model[i].enumLiteral;
					}
					$scope.availableSearchParams.push(param);
				}
			}

			// add option for complex query
			var param = {};
			param.key = "QUERY_ALL";
			param.name = "Any";
			param.type = "ATTR_ALL";
			param.label = "Search All";
			param.placeholder = 'will search any text field...';
			$scope.availableSearchParams.push(param);

			$scope.searchOptions = {};
			if ($scope.maxResults != null) {
				$scope.searchOptions.maxResults = $scope.maxResults;
			} else {
				$scope.searchOptions.maxResults = 10;
			}
			// default orderBy to the first field (typically id)
			$scope.searchOptions.orderBy = $scope.availableSearchParams[0].key;
			$scope.searchOptions.ascending = false;
			//console.log("*&* getCases buildParams" + $scope.searchOptions.ascending);
			getCases(0);

		}

		getCases = function (startPos) {
			SrcAppBPMServices.findCaseRefByCriteria($scope.selectedClass.name, $scope.selectedClass.appDetails.majorVersion, $scope.searchParams, $scope.availableSearchParams, $scope.searchOptions, startPos).then(
					function (result) {
						for (var caseRef in result.caseReference) {
							$scope.caseRefs.caseReference.push(result.caseReference[caseRef]);
						}
						$scope.caseRefs.hasMoreResults = result.hasMoreResults;
						//console.log("*&* Result: " + result);
						$scope.loaded = true;
						//console.log("Loaded");
					},
					function (error) {
						//window.cap.LOGGER.error("ERR_CUST_CLIENT_CTLR_GET_CASE_MODEL: " + error);
					}
			);
		}

		//retrieve case model
		$scope.selectedType = function (data) {
			//console.log("*&* called selectedType, reset searchParams from ", $scope.searchParams);
			$scope.loaded = false;
			$scope.searchParams = {};
			$scope.caseRefs.caseReference = [];
			SrcAppBPMServices.getCaseModel(data.name).then(
					function (result) {
						$scope.casemodels = result.caseClassInfo;
						// currently returns all case models. Find the one we want:
						// loop to find the case model
						for (var i = 0; i < $scope.casemodels.length; i++) {
							//console.log("*&* loop iteration " + i + " " + $scope.casemodels[i].name + " is same as " + data.name + "?");
							if ($scope.casemodels[i].name === data.name) {
								$scope.casemodel = $scope.casemodels[i].attributeInfo;
							}
						}
						// now build the config for search
						buildSearchParams($scope.casemodel);
					},
					function (error) {
						window.cap.LOGGER.error("ERR_CUST_CLIENT_CTLR_GET_CASE_MODEL: " + error);
					}
			);
		}

		$scope.getMoreCases = function() {
			//console.log("*&* getCases more");
			getCases($scope.caseRefs.caseReference.length +1);
		}
		//get the case data when the user puts in search params
		$scope.$on('advanced-searchbox:modelUpdated', function (event, model) {
			if (isEmpty($scope.searchParams)) {
				$scope.caseRefs.caseReference = [];
			}
			if ($scope.search) {
				$scope.caseRefs.caseReference = [];
				//console.log("*&* getCases modelUpdated");
				getCases(0);
			}
			/*if (!isEmpty($scope.searchParams)) {
			 //console.log("Sending search");
			 $scope.caseRefs.caseReference = [];
			 getCases();
			 }*/
		});

		$scope.$on('advanced-searchbox:enteredEditMode', function (event, searchParameter) {
			$scope.search = false;
		});

		$scope.$on('advanced-searchbox:removedAllSearchParam', function (event) {
			$scope.searchParams = {};
			$scope.caseRefs.caseReference = [];
		});

		$scope.$on('advanced-searchbox:leavedEditMode', function (event, searchParameter) {
			////console.log("Sending search");
			$scope.search = true;
		});

		$scope.$on('advanced-searchbox:removedSearchParam', function (event, searchParameter) {
		});



		$scope.$watch('searchOptions.maxResults', function (newVal, oldVal) {
			//console.log("watch:" + newVal + " " + oldVal);
			if ($scope.loaded && ($scope.searchOptions.maxResults > 0) && (newVal != oldVal)) {
				$scope.caseRefs.caseReference = [];
				//console.log("*&* getCases maxResults");
				getCases(0);
			}
		})

		$scope.$watch('searchOptions.ascending', function (newVal, oldVal) {
			//console.log("watch:" + newVal + " " + oldVal);
			if (($scope.loaded) && (newVal != oldVal)) {
				$scope.caseRefs.caseReference = [];
				//console.log("*&* getCases ascending");
				getCases(0);
			}
		})

		$scope.$watch('searchOptions.orderBy', function (newVal, oldVal) {
			//console.log("watch:" + newVal + " " + oldVal);
			if (($scope.loaded) && (newVal != oldVal)) {
				$scope.caseRefs.caseReference = [];
				//console.log("*&* getCases orderBy");
				getCases(0);
			}
		})


}]);