/**
 * Created by JeremyS on 08/12/16.
 */

angular.module('components.workItemsGrid', ['angular-sortable-view', 'smart-table', 'lrDragNDrop'])
    .controller('WorkItemsGridController', ['$scope', '$rootScope', 'CaseDisplayBPMServices', '$filter', '$location', function ($scope, $rootScope, CaseDisplayBPMServices, $filter, $location) {
        //console.log("*** Work Items Grid Controller");

        $scope.allColumns = ['appInstanceDescription','name','activityID','activityName','deadline','distributionStrategy','groupID','appID','appInstance','appName','scheduleStatus','version','visible','workTypeDescription','workTypeID','workTypeVersion','description','state','id','priority','startDate','endDate', "attribute1","attribute2","attribute3","attribute4","attribute5","attribute6","attribute7","attribute8","attribute9","attribute10","attribute11","attribute12","attribute13","attribute14","attribute15","attribute16","attribute17","attribute18","attribute19","attribute20","attribute21","attribute22","attribute23","attribute24","attribute25","attribute26","attribute27","attribute28","attribute29","attribute30","attribute31","attribute32","attribute33","attribute34","attribute35","attribute36","attribute37","attribute38","attribute39","attribute40"];

        $scope.getViewDetails = function(viewId) {
            // retrieve view details from server and store on scope
            //console.log("***grid*** Getting view: " + viewId);
           CaseDisplayBPMServices.getViewColumnDetails(viewId).then(
               function(response) {
                       ////console.log("*grid - got case view details: " + JSON.stringify(response));
                       $scope.workViewDefinition = response;
                   })
        }


        // for each column - get the proper column name
        $scope.getHeaderForColumn = function(columnName) {
            var value;
            if ($rootScope.wlFacade[columnName] != null) {
                value = $rootScope.wlFacade[columnName];
            } else {
                // if nothing in facade then just return columnName
                value = columnName;
            }
            return value;
        }

        $scope.getHeaderForColumnSort = function(columnName) {
            return columnName;
        }

        $scope.setWorkListColumns = function() {
            var workListColumns = [ "id", "activityName", "startDate", "priority"];
            CaseDisplayBPMServices.readWlColumnSettings($rootScope.userResourceDetails.guid).then(
                function(response) {
                    if (response != '') {
                        workListColumns = response;
                    } else {
                        // nothing stored - show default from config file
                        var workListColumns = $rootScope.wlDefaults.columns;
                    }
                    $scope.workListColumns = workListColumns;
                    $scope.selectedWlColumns = workListColumns;

                    // setup selected columns so we can pick them
                    $scope.columnConfig = {
                        availableColumns: $($scope.allColumns).not($scope.workListColumns).get(),
                        selectedColumns: $scope.workListColumns
                    }
                })
        }

        $scope.resetToDefault = function() {
            // resets the wlColumns to the default from config file
            $scope.workListColumns = $rootScope.wlDefaults.columns;
            $scope.columnConfig = {
                availableColumns: $($scope.allColumns).not($scope.workListColumns).get(),
                selectedColumns: $scope.workListColumns
            }
            $scope.persistWLColumns();
        }

        $scope.toggleWLConfig = function() {
            $scope.configureWl = !$scope.configureWl;
        }

        $scope.columnConfigDrop = function() {
            // triggering this on sort change rather than drop because this one is triggered after $scope.workListColumns has been updated
            $scope.persistWLColumns();
        }

        $scope.persistWLColumns = function() {
            // save the worklist column layout to BPM user preferences
            CaseDisplayBPMServices.persistWlColumnSettings($rootScope.userResourceDetails.guid, $scope.workListColumns);
        }

        $scope.getCaseRefsForWI = function(task) {
            if (task.caseRefs != undefined) {
                //return task.caseRefs;
            }
            else {
                task.caseRefs = {};
                taskId = task.id.id;
                if (typeof taskId == 'string') {
                    // have to call the API to preview the workitem and get any Case Refs
                    CaseDisplayBPMServices.previewWorkItemFromList(taskId, $rootScope.userResourceDetails.guid).then(
                        function (result) {
                            // pull the caseRefs
                            var preview = result.workItemPreview[0].FieldPreview;
                            var refs = [];
                            for (i = 0; i < preview.length; i++) {
                                var curField = preview[i];
                                if (curField.type == "Data Reference") {
                                    var ref = {};
                                    ref.caseRef = curField.simpleSpec.value[0];
                                    ref.title = formatCaseReference(curField.simpleSpec.value[0]);
                                    refs.push(ref);
                                }
                            }
                            task.caseRefs = refs;
                        }
                    )
                }
            }
        }

        formatCaseReference = function(caseRef) {
            // Assign case class details
            caseRefParsed = caseRef.split("-");
            caseClass = caseRefParsed[2];
            // Get the short class name - caseType
            caseClassParsed = caseClass.split(".");
            caseType = caseClassParsed[caseClassParsed.length - 1];
            caseId = caseClass = caseRefParsed[3];
            return caseType + ': ' + caseId;
        }

        $scope.wlDropSuccess = function() {
            // we got a drop on the default worklist so persist the current order to the settings service
            $scope.persistWLColumns();
        }

        $scope.openWi = function(task) {
            // currently using rootScope.wiSelection to trigger the WI change so this works both for grid and OOTB task lists
            /*if (task.caseRefs.length < 1) {
                alert("Case Manager only supports opening workitems that are related to cases");
            } else {
                if (task.caseRefs > 1) {
                    console.warn("The task is related to multiple cases - we will open within the context of the first case reference: " + task.caseRefs[0].title);
                }
                $location.path("/pageCaseDisplay/" + task.caseRefs[0].caseRef + "/" + task.id.id);
            }*/
            if (!$scope.justCase) {
                var tasks = [];
                tasks.push(task);
                $rootScope.wiSelection = tasks;
            }

        }

        $scope.openCase = function (caseRef) {
            $scope.justCase = true;
            $location.path("/pageCaseDisplay/"+caseRef);
        }

        // for each column - get the data from the right part of the WI data set
        $scope.getValueForColumn = function(columnName, task) {
            var value;
            if (columnName.substring(0,9) != 'attribute') {
                // system attribute so return the right part of the data model
                switch (columnName) {
                    case 'appInstanceDescription':
                        value = task.data.header.itemContext.appName;
                        break;
                    case 'name':
                        value = task.name;
                        break;
                    case 'activityID':
                        value = task.data.header.itemContext.activityID;
                        break;
                    case 'activityName':
                        value = task.data.header.itemContext.activityName;
                        break;
                    case 'deadline':
                        if (task.remaining.str != undefined) {
                            value = task.remaining.num + ' ' + task.remaining.str;
                        }
                        break;
                    case 'distributionStrategy':
                        value = task.data.header.distributionStrategy;
                        break;
                    case 'groupID':
                        value = '?';
                        break;
                    case 'appID':
                        value = task.data.header.itemContext.appID;
                        break;
                    case 'appInstance':
                        value = task.data.header.itemContext.appInstance;
                        break;
                    case 'appName':
                        value = task.data.header.itemContext.appName;
                        break;
                    case 'scheduleStatus':
                        value = task.data.header.flags.scheduleStatus;
                        break;
                    case 'version':
                        value = task.data.id.version;
                        break;
                    case 'visible':
                        value = task.data.visible;
                        break;
                    case 'workTypeDescription':
                        value = '';
                        break;
                    case 'workTypeID':
                        value = '';
                        break;
                    case 'workTypeVersion':
                        value = '';
                        break;
                    case 'description':
                        value = task.description;
                        break;
                    case 'state':
                        value = task.data.state;
                        break;
                    case 'id':
                        value = task.id.id;
                        break;
                    case 'priority':
                        value = task.priority;
                        break;
                    case 'startDate':
                        value = $filter('date')(task.data.header.startDate, 'medium');
                        break;
                    case 'endDate':
                        value = $filter('date')(task.target, 'medium');
                        break;
                    default:
                        value = '?';
                }
            } else {
                // just return the attribute data
                if (task.attributesData != null) {
                    // format date attributes
                    if (columnName == 'attribute6' || columnName == 'attribute7' || columnName == 'attribute19' || columnName == 'attribute20') {
                        value = $filter('date')((task.attributesData[columnName]), 'medium');
                    } else {
                        value = task.attributesData[columnName];
                    }
                } else {
                    value = '';
                }
            }
            return value;
        }

    }]);