var appModuleName = "CaseManager";

// This is a callBack when the TIBCO Libs are finally loaded
function onOCFAppLoad(moduleName) {
    return [appModuleName];
}

// Note tibco-components.min.css and tibco-components.min.js are pulled from the product CaseManagementApp sample.
// You should download this from downloads site and deploy it first.
function collectDependencies() {
    cap.depend("ComponentShowcase", "css/CaseManagement.css", false, false);
    cap.depend("tibco-components", "../app-cdn/tcf/tibco-components.min.css", false, false);
    cap.depend("tibco-components", "../app-cdn/tcf/tibco-components.min.js", false, false);
}


function tcfModuleLoad() {
    tcf.BPMTranslationServiceConfig.registerApp("CaseManager", "tcf");
}


// Each directive used as a section needs to be added here to the dependency chain - eg. components.sections.caseSection.directive
var CaseManager = angular.module("CaseManager", ['TibcoFramework', 'ngNewRouter' ,'components.sections.caseSection.directive', 'components.sections.analyticsSection.directive', 'components.sections.tasksSection.directive', 'components.nimbusDisplay.directive', 'components.orgbrowserDisplay.directive', 'components.businessServiceMenu.directive', 'components.pageAnalytics', 'components.pageBusinessService', 'components.pageCaseDisplay', 'components.pageCaseSearch', 'components.pageHome', 'components.pageNimbus', 'components.pageOrgModel', 'CaseManagementApp.pageRules', 'components.pageWorkList', 'components.runBusinessService.directive', 'components.webChatBot', 'components.webChatBot.directive'])
//.config(function($compileProvider) {
//  $compileProvider.preAssignBindingsEnabled(true);
//}


// NOTE: After cloning the app, if you want to use your own L10N bundle,
// uncomment the following code and pass the right app name
CaseManager.run(['BPMTranslationServiceConfig', function (BPMTranslationServiceConfig) {
    BPMTranslationServiceConfig.registerApp("CaseManager", null);
    BPMTranslationServiceConfig.unregisterApp("workapp", null);
    //console.log("***** Registered CaseManager with translation service");
}]);


CaseManager.controller("CaseManagerCtrl", function ($scope, $rootScope, $mdSidenav, $http, BPMLoginService, CaseManagementService, $timeout, $router, $location, BPMRolesService, $interval, BPMWorklistService) {

    /*********************************************DECLARATION OF FUNCTIONS **************************************/

    //function to change style
    //TODO NAM we could introspect theme but I tried earlier was not obvious
    $scope.changeTheme = function () {
        //console.log("************************************************************ Change actual theme : " + $scope.dynamicColorTheme);
        if (typeof $scope.dynamicColorTheme == 'undefined') {
            $scope.dynamicColorTheme = 'default';
        } else if ($scope.dynamicColorTheme == 'default') {
            $scope.dynamicColorTheme = 'metal';
        } else if ($scope.dynamicColorTheme == 'metal') {
            $scope.dynamicColorTheme = 'redAndGrey';
        } else if ($scope.dynamicColorTheme == 'redAndGrey') {
            $scope.dynamicColorTheme = 'default';
        } else {
            $scope.dynamicColorTheme = 'default';
        }
    };

    //Function that change the menu layout from top to side
    $scope.changeMenuLayout = function () {
        if ($scope.dynamicMenuLayout == "side") {
            $scope.dynamicMenuLayout = "top";
        } else {
            $scope.dynamicMenuLayout = "side";
        }
    };


    // function to handle logout/redirect
    $scope.logout = function () {


        var loginModel;
        //alert("$scope.logoutRedirection : " + $scope.logoutRedirection);
        if ($scope.logoutRedirection=="home"){
            //alert("GO TO HOME");
            loginModel = {"logoutRedirect": "components/login/login.html?tcf-redirect=" + encodeURIComponent(window.location.href.split('#')[0])};
        } else {
            loginModel = {"logoutRedirect": "components/login/login.html?tcf-redirect=" + encodeURIComponent(window.location.href)};
        }
        BPMLoginService.logout(loginModel, {
            onSuccess: function () {
            }
        });
    };

    // function to transform camel case to human readable - this is used when we want to display some data from the BPM models that doesnt have a label - so it is more readable
    $scope.camelToHuman = function (data) {
        return data.replace(/([A-Z]+)/g, ' $1').replace(/^./, function (str) {
            return str.toUpperCase();
        })
    };


    $scope.getExtendedInfos = function () {
        //GET ExtendedInfos
        BPMLoginService.getExtendedUserInfo({
            onSuccess: function (xmlResourceDetails) {
                $scope.userResourceDetails = xmlResourceDetails;
                $rootScope.userResourceDetails = xmlResourceDetails;

                // alert("Setting Guid to " + $scope.userResourceDetails.guid);

                //TODO check if we still need Session Service
                //SessionService.setGuid($scope.userResourceDetails.guid);


                //We build the menu, depending of who the user is (if config "useRoles" is configured to true )
                if ($scope.useRoles == "true") {
                    BPMRolesService.getRolesForUser($scope.userResourceDetails, $scope.setMenu);
                } else {
                    $scope.setMenu();
                }

                // stop session from timing out (uses config value for keepAlive)
                if ($scope.configData.session.keepAlive > 0) {
                    $interval(function () {
                        var worklistServiceModel = {};
                        worklistServiceModel.templateData = {};
                        var start = '0';
                        var count = '0';
                        var entityType = 'RESOURCE';
                        var entityGuid = $rootScope.userResourceDetails.guid;
                        BPMWorklistService.getWorklistItems(entityType,entityGuid,start,count, worklistServiceModel, {
                            useDefault: false,
                            onSuccess: function (userInfo) {
                                //console.log("KeepAlive!@" + Date());
                            },
                            onFailure: function (error) {
                                console.error("KeepAlive failed: Is BPM Engine running?");
                            }
                        });
                    }, $scope.configData.session.keepAlive * 1000 * 60);
                }




            }
        });

    };

    // Helper method to get component settings from the BPMRoleService
    $scope.getAppSetting = function ( settingName) {
        //alert("GetAppSetting : " + settingName);
        return (BPMRolesService.getSettingForApp("/apps/" + appModuleName, settingName));
    };

    $scope.setMenu = function () {

        var defaultMenuPages =  [{
                                    'id': 'pageHome',
                                    'labelKey': 'home',
                                    'icon': 'svg/home_icon.svg',
                                    'page': 'pageHome'
                                },{
                                    'id': 'pageCaseSearch',
                                    'labelKey': 'case',
                                    'icon': 'svg/cases_icon.svg',
                                    'page': 'pageCaseSearch'
                                },
                                {
                                    'id': 'pageWorkList',
                                    'labelKey': 'task',
                                    'icon': 'svg/work_list_icon.svg',
                                    'page': 'pageWorkList'
                                },
                                {
                                    'id': 'pageAnalytics',
                                    'labelKey': 'analytics',
                                    'icon': 'svg/analytics.svg',
                                    'page': 'pageAnalytics'
                                },
                                {
                                    'id': 'pageOrgModel',
                                    'labelKey': 'resources',
                                    'icon': 'svg/org_structure.svg',
                                    'page': 'pageOrgModel'
                                },
                                {
                                    'id': 'pageNimbus',
                                    'labelKey': 'processes',
                                    'icon': 'svg/nimbus.svg',
                                    'page': 'pageNimbus'
                                }];



        if ($scope.useRoles == "true") {
            //console.log('************************CHEKING ROLES**********************************');

            var tmpMenuPages = [];
            for (var i=0;  i<defaultMenuPages.length;i++){
                var page = defaultMenuPages[i];
                //console.log('************************LabelKey : '+ page.labelKey);
                var pageRight = $scope.getAppSetting(page.labelKey + "Show");
                //console.log('************************Right : '+ pageRight);
                if (pageRight == "true") {
                    tmpMenuPages.push(page);
                }
            }
            $scope.menuPages = tmpMenuPages;

            /* TODO : this is not handling start page anymore => we use routing page
             THis could be readded to handle default routing page
             care full now the selected section menu button might not be selected*/
            //$scope.sideBarSelect($scope.menuPages[0].id, $scope.menuPages[0].page);
        } else {
            $scope.menuPages = defaultMenuPages;


            /* TODO : this is not handling start page anymore => we use routing page
             THis could be readded to handle default routing page
             care full now the selected section menu button might not be selected*/
            //$scope.sideBarSelect($scope.configData.startPage.pageSelected, $scope.configData.startPage.pageSelected);
        }
        $scope.appReady = true;
        $scope.$apply();

    };


    $scope.getContifg = function () {
        // read config file
        $http.get("CaseManager.config.json")
            .then(function (response) {
                $scope.configData = response.data;
                // if($scope.configData["selectedCaseModel"].length>0)

                if ($scope.configData["selectedCaseModel"]["appName"].length > 0) {
                    $scope.appCaseModel = $scope.configData["selectedCaseModel"];
                } else {
                    CaseManagementService.configCaseModelAlert();
                }
                if ($scope.configData["analyticsConfig"]["tableName"].length > 0) {
                    $scope.analyticsConfig = $scope.configData["analyticsConfig"];
                }
                if ($scope.configData["caseDisplayConfig"]["useCustomTemplates"].length > 0) {
                    $scope.caseDisplayConfig = $scope.configData["caseDisplayConfig"];
                }
                if ($scope.configData["businessServiceConfig"]["caseStartCategories"].length > 0) {
                    $scope.businessServiceConfig = $scope.configData["businessServiceConfig"];
                }
                if ($scope.configData["nimbusConfig"]["baseurl"].length > 0) {
                    $scope.nimbusConfig = $scope.configData["nimbusConfig"];
                }
                if ($scope.configData["wlFacade"]["attribute1"].length > 0) {
                    $scope.wlFacade = $scope.configData["wlFacade"];
                    $rootScope.wlFacade = $scope.configData["wlFacade"];
                }
                if ($scope.configData["wlDefaults"]["columns"].length > 0) {
                    $scope.wlDefaults = $scope.configData["wlDefaults"];
                    $rootScope.wlDefaults = $scope.configData["wlDefaults"];
                }
                if ($scope.configData["casePendingList"]["notificationsEnabled"].length > 0) {
                    $scope.casePendingList = $scope.configData["casePendingList"];
                    $rootScope.casePendingList = $scope.configData["casePendingList"];
                }
                if ($scope.configData["session"]["keepAlive"].length > 0) {
                    $scope.session = $scope.configData["session"];
                }

                //Getting colorTheme from config file

                //TODO NAM have to understand Jez code to be defensive or not
                
                $scope.dynamicColorTheme = $scope.configData.colorTheme.selected;
                $scope.dynamicMenuLayout = $scope.configData.layout.menuSelected;
                $scope.useRoles = $scope.configData.role.useRoles;

                $scope.homePageConf = $scope.configData.homePage;
                $scope.logoutRedirection = $scope.configData.navigation.logoutRedirection;


                $scope.getExtendedInfos();

            });
    };


    //ROUTING Configuration
    $router.config([
        {path: '/', component: 'pageHome'},
        {path: '/pageHome', component: 'pageHome'},
        {path: '/pageBusinessService', component: 'pageBusinessService'},
        {path: '/pageCaseDisplay/:selectedCaseRef', component: 'pageCaseDisplay'},
        {path: '/pageCaseDisplay/:selectedCaseRef/:selectedWorkItemId', component: 'pageCaseDisplay'},
        {path: '/pageWorkList', component: 'pageWorkList'},
        {path: '/pageWorkList', component: 'pageWorkList'},
        {path: '/pageOrgModel', component: 'pageOrgModel'},
        {path: '/pageAnalytics', component: 'pageAnalytics'},
        {path: '/pageAnalytics2', component: 'pageAnalytics2'},
        {path: '/pageCaseSearch', component: 'pageCaseSearch'},
        {path: '/pageNimbus', component: 'pageNimbus'}

    ]);



    // handle selection of a side bar items
    //TODO NAM JEZ can I go back to one parameter as it is the same now... just wondering if you use if (data == 'default') {
    $scope.sideBarSelect = function (data, page) {
        //console.log("*** switchPane: " + data + " with page: " + page);
        // this is to clear the state of the task section which has to use $root
        $rootScope.wiSelection = undefined;
        $rootScope.wiComplete = undefined;
        if (data == 'default') {
            //console.log("*** setting default");
            $scope.selectedSideBarAction = "C";
            $location.path("/");

            //$scope.$apply();
        } else {
            if ($scope.selectedSideBarAction != data) {
                $scope.selectedSideBarAction = data;
                $location.path("/" + page);
            } else {
                //TODO JEZ NAM what is happening here when reclick ? it is a refresh
                // same icon clicked again
                // yes this is a hack for a refresh
                //console.log("re-clicked: " + data);
                $scope.selectedSideBarAction = "NA";

                $timeout(function () {
                    $scope.selectedSideBarAction = data;
                    $location.path("/" + page);
                }, 0);
            }
        }
    };


    $scope.backToLastCase = function () {
        //alert("Last CaseRef : " + $rootScope.lastCaseRef);
        if ($rootScope.lastCaseRef == undefined) {
            alert("No recent case visited");
        } else {
            $scope.sideBarSelect('pageCaseSearch', "pageCaseDisplay/" + $rootScope.lastCaseRef);
        }

    };


    //required for localization/languages
    $rootScope.appConfig = {
        // JS: to make grid WL work... bug in Appdev?
        // appId: "CaseManager"
        appId: "tcf"
    };


    /*********************************************IMPLEMENTATION OF CONTROLLER **************************************/


    // get user info such as GUID - if not logged in, redirect
    $scope.authenticated = false;
    $scope.appReady = false;
    BPMLoginService.getUserInfo({
        onSuccess: function (userInfo) {

            if (userInfo && userInfo.userName) {
                $scope.authenticated = true;
                $scope.userInfo = userInfo;
                $scope.profile = {};
                $scope.profile.userName = userInfo.userName;
                $scope.profile.userNameImg = "icons/users/" + userInfo.userName +   ".png";
                //console.log("********************Authenticated, we need to load the config");
                $scope.getContifg();

            } else {
                window.location.replace("components/login/login.html?tcf-redirect=" + encodeURIComponent(window.location.href));
            }
        }
    });


});


// Themes
//check palettes : http://www.google.com/design/spec/style/color.html#color-color-palette
CaseManager.config(function ($mdThemingProvider) {

    var customRed = $mdThemingProvider.extendPalette('red', {
        '700': 'e1081d',

    });
    // Register the new color palette map with the name <code>neonRed</code>
    $mdThemingProvider.definePalette('customRed', customRed);


    var customRed2 = $mdThemingProvider.extendPalette('red', {
        '700': 'C12017',

    });
    // Register the new color palette map with the name <code>neonRed</code>
    $mdThemingProvider.definePalette('customRed2', customRed2);




    var customGreen = $mdThemingProvider.extendPalette('green', {
        '700': '4CB28F',

    });
    // Register the new color palette map with the name <code>neonRed</code>
    $mdThemingProvider.definePalette('customGreen', customGreen);


    var blueWithBlack = $mdThemingProvider.extendPalette('blue', {
        '900': '#000000'
    });


    $mdThemingProvider.definePalette('blueWithBlack', blueWithBlack);

    var indigoWithTibcoBlue = $mdThemingProvider.extendPalette('indigo', {
        '900': '#3E8DDD'
    });


    $mdThemingProvider.definePalette('indigoWithTibcoBlue', indigoWithTibcoBlue);

    $mdThemingProvider.theme('default')
        .primaryPalette('blueWithBlack', {
            'default': '700',
            'hue-2': '900'
        })
        .accentPalette('indigoWithTibcoBlue', {
            'default': '400', // use shade 200 for default, and keep all other shades the same
            'hue-2': '900'
        })
        .warnPalette('red')
        .backgroundPalette('grey', {
            'default': 'A100'
        })
    /*.foregroundPalette['3'] = 'rgba(255,255,255,1)';*/


    $mdThemingProvider.theme('metal')
        .primaryPalette('blue-grey')
        .accentPalette('orange', {
            'default': '400',// use shade 200 for default, and keep all other shades the same
            'hue-3': 'A100'
        })
        .warnPalette('red')
        .backgroundPalette('grey', {
            'default': 'A100'
        });

    $mdThemingProvider.theme('redAndGrey')
        .primaryPalette('customRed', {
            'default': '700',
            'hue-1': '200'
        })
        .accentPalette('grey', {
            'default': '700' // use shade 200 for default, and keep all other shades the same
        })
        .warnPalette('red')
        .backgroundPalette('grey', {
            'default': 'A100'
        });


    $mdThemingProvider.theme('greyAndRed')
        .primaryPalette('grey', {
            'default': '900',
            'hue-1': '900',
            'hue-2': '300',
            'hue-3': '500'
        })
        .accentPalette('customRed2', {
            'default': '900', // use shade 200 for default, and keep all other shades the same
            'hue-2': '700'
        })
        .warnPalette('red')
        .backgroundPalette('grey', {
            'default': 'A100'
        });



    $mdThemingProvider.theme('greenAndGrey')
        .primaryPalette('customGreen', {
            'default': '700',
            'hue-1': '200'
        })
        .accentPalette('grey', {
            'default': '700' // use shade 200 for default, and keep all other shades the same
        })
        .warnPalette('red')
        .backgroundPalette('grey', {
            'default': 'A100'
        });


    // to set a different theme - change below to one of the theme names above - eg. metal
    // $mdThemingProvider.setDefaultTheme('metal');
    $mdThemingProvider.setDefaultTheme('default');

    $mdThemingProvider.alwaysWatchTheme(true);
});
CaseManager.filter('daysaway', function () {
    return function (date) {
        return moment(date).fromNow();
    };
});

CaseManager.filter('dateDiffFromNow', function () {
    return function (date) {
        return moment(date).diff(moment(), 'days');
    };
});

CaseManager.config(function($mdIconProvider) {
        // Register svg icons with short names
        $mdIconProvider.icon('close:black', 'icons/ic_close_black_24px.svg')
        $mdIconProvider.icon('close:white', 'icons/ic_close_white_24px.svg')
        $mdIconProvider.icon('action:white', 'icons/ic_action_white_24px.svg')
        $mdIconProvider.icon('close:none', 'icons/ic_close_24px.svg')
        $mdIconProvider.icon('archive:white', 'icons/ic_archive_white_24px.svg')
        $mdIconProvider.icon('audit:white', 'icons/ic_audit_white_24px.svg')
        $mdIconProvider.icon('document:white', 'icons/ic_document_white_24px.svg')
        $mdIconProvider.icon('draghandle:black', 'icons/ic_drag_handle_black_24px.svg')
        $mdIconProvider.icon('expandmore:black', 'icons/ic_expand_more_black_24px.svg')
        $mdIconProvider.icon('fileupload:black', 'icons/ic_file_upload_24px.svg')
        $mdIconProvider.icon('folder:white', 'icons/ic_folder_white_24px.svg')
        $mdIconProvider.icon('info:white', 'icons/ic_info_white_24px.svg')
        $mdIconProvider.icon('link:white', 'icons/ic_link_white_24px.svg')
        $mdIconProvider.icon('menu:white', 'icons/ic_menu_white_24px.svg')
        $mdIconProvider.icon('edit:white', 'icons/ic_mode_edit_white_24px.svg')
        $mdIconProvider.icon('nosearch:none', 'icons/ic_nosearch_24px.svg')
        $mdIconProvider.icon('reply:black', 'icons/ic_reply_black_24px.svg')
        $mdIconProvider.icon('search:white', 'icons/ic_search_white_24px.svg')
        $mdIconProvider.icon('settings:black', 'icons/ic_settings_black_24px.svg')
        $mdIconProvider.icon('tasks:white', 'icons/ic_tasks_white_24px.svg')
        $mdIconProvider.icon('helpoutline:black', 'icons/ic_help_outline_black_24px.svg')
        $mdIconProvider.icon('social:white', 'icons/ic_social_white_24px.svg')

    // action icons

        $mdIconProvider.icon('actiondefault:black', 'icons/actionType/default-action-icon.svg')

    // adhoc icons

        $mdIconProvider.icon('adhocdefault:black', 'icons/adhocType/default-adhoc-icon.svg')

    // audit icons

        $mdIconProvider.icon('BDS_GLOBAL_DATA_CASE_STATE_CHANGE', 'icons/audit/BDS_GLOBAL_DATA_CASE_STATE_CHANGE.svg')
        $mdIconProvider.icon('BDS_GLOBAL_DATA_CREATE_CASE', 'icons/audit/BDS_GLOBAL_DATA_CREATE_CASE.svg')
        $mdIconProvider.icon('CMF_CASE_ACTION_COMPLETED', 'icons/audit/CMF_CASE_ACTION_COMPLETED.svg')
        $mdIconProvider.icon('CMF_CASE_ACTION_STARTED', 'icons/audit/CMF_CASE_ACTION_STARTED.svg')
        $mdIconProvider.icon('CMF_CASE_ADHOC_TASK_CANCELLED', 'icons/audit/CMF_CASE_ADHOC_TASK_CANCELLED.svg')
        $mdIconProvider.icon('CMF_CASE_ADHOC_TASK_STARTED', 'icons/audit/CMF_CASE_ADHOC_TASK_STARTED.svg')
        $mdIconProvider.icon('CMF_CASE_TASK_CANCEL', 'icons/audit/CMF_CASE_TASK_CANCEL.svg')
        $mdIconProvider.icon('CMF_CASE_TASK_CLOSE', 'icons/audit/CMF_CASE_TASK_CLOSE.svg')
        $mdIconProvider.icon('CMF_CASE_TASK_COMPLETE', 'icons/audit/CMF_CASE_TASK_COMPLETE.svg')
        $mdIconProvider.icon('CMF_CASE_TASK_OPEN', 'icons/audit/CMF_CASE_TASK_OPEN.svg')

    })

    .run(function($templateRequest){

        // Pre-fetch icons sources by URL and cache in the $templateCache...
        // subsequent $templateRequest calls will look there first (eg. from md-icon).

        var urls = [
            'icons/ic_close_black_24px.svg',
            'icons/ic_social_white_24px.svg',
            'icons/ic_action_white_24px.svg',
            'icons/ic_close_white_24px.svg',
            'icons/ic_close_24px.svg',
            'icons/ic_archive_white_24px.svg',
            'icons/ic_audit_white_24px.svg',
            'icons/ic_document_white_24px.svg',
            'icons/ic_drag_handle_black_24px.svg',
            'icons/ic_expand_more_black_24px.svg',
            'icons/ic_file_upload_24px.svg',
            'icons/ic_folder_white_24px.svg',
            'icons/ic_info_white_24px.svg',
            'icons/ic_link_white_24px.svg',
            'icons/ic_menu_white_24px.svg',
            'icons/ic_mode_edit_white_24px.svg',
            'icons/ic_nosearch_24px.svg',
            'icons/ic_reply_black_24px.svg',
            'icons/ic_search_white_24px.svg',
            'icons/ic_settings_black_24px.svg',
            'icons/ic_tasks_white_24px.svg',
            'icons/ic_help_outline_black_24px.svg',
            'icons/actionType/default-action-icon.svg',
            'icons/adhocType/default-adhoc-icon.svg',
            'icons/audit/BDS_GLOBAL_DATA_CASE_STATE_CHANGE.svg',
            'icons/audit/BDS_GLOBAL_DATA_CREATE_CASE.svg',
            'icons/audit/CMF_CASE_ACTION_COMPLETED.svg',
            'icons/audit/CMF_CASE_ACTION_STARTED.svg',
            'icons/audit/CMF_CASE_ADHOC_TASK_CANCELLED.svg',
            'icons/audit/CMF_CASE_ADHOC_TASK_STARTED.svg',
            'icons/audit/CMF_CASE_TASK_CANCEL.svg',
            'icons/audit/CMF_CASE_TASK_CLOSE.svg',
            'icons/audit/CMF_CASE_TASK_COMPLETE.svg',
            'icons/audit/CMF_CASE_TASK_OPEN.svg'
        ];

        angular.forEach(urls, function(url) {
            $templateRequest(url);
        });

    });




