/**
 * Created by NamOs on 08/09/15.
 * Updated by JeremyS on 02/01/16.
 */

var bpmCaseDisplay = angular.module('components.bpmCaseDisplay', []);
bpmCaseDisplay.service('CaseDisplayBPMServices', function(RestApiService, $q, $timeout, $http) {

	var getBaseUrl = function() {
		var baseUrl = ((window.location.protocol.lastIndexOf('//') == window.location.protocol.length - 1)
				? window.location.protocol : (window.location.protocol + '//'))
			+ ((window.location.host.lastIndexOf('/') == (window.location.host.length - 1))
				? window.location.host : (window.location.host + '/'));
		return baseUrl;
	};
	var getBPMRestUrl = function() {
		var baseUrl = getBaseUrl() + 'bpm/rest/';
		return baseUrl;
	};
    var getAppsBaseUrl = function() {
        var appsBaseUrl = getBaseUrl() + 'apps/rest/';
        return appsBaseUrl;
    };
	var getHeaders = function() {
		//return {'Authorization': "Basic dGliY28tYWRtaW46c2VjcmV0"};
		return {};
	};
	var isCorsEnabled = function() {
		return true;
	};
	var restServiceConfig = {
		getBaseUrl : getBPMRestUrl,
        getAppsBaseUrl : getAppsBaseUrl,
		headers : getHeaders(),
		isCorsEnabled : isCorsEnabled
	};

	var callback = function(deferred) {
		this.onSuccess = function(result) {
			deferred.resolve(result);
		},
		this.onFailure = function(message) {
				alert("Failed to call BPM API: " + message);
				deferred.reject(eval('(' + '{ message: \"' + message + '\" }' + ')') );
			};
	};
	
	this.listAdhocActivitiesByGlobalRef = function(caseReference) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ListAdhocActivitiesByGlobalRefRequest(caseReference);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getProcessManagementService().listAdhocActivitiesByGlobalRef(request, new callback(deferred));
		return deferred.promise;
	};

	this.runAdhocActivity = function(processId, activityId) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.RunAdhocActivityRequest(processId, activityId);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getProcessManagementService().runAdhocActivity(request, new callback(deferred));
		return deferred.promise;
	};

	this.cancelAdhocActivity = function(processId, activityId) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.CancelAdhocActivityRequest(processId, activityId);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getProcessManagementService().cancelAdhocActivity(request, new callback(deferred));
		return deferred.promise;
	};

	this.getNavigateDetailed = function(caseRef) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.NavigateCaseDetailedRequest(caseRef);
		request.includeSummary = true;
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getGlobaldataService().navigateCaseDetailed(request, new callback(deferred));
		return deferred.promise;
    };

	this.listCaseAction = function(caseRef) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ListCaseActionRequest(caseRef);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getCaseActionService().listCaseAction(request, new callback(deferred));
		return deferred.promise;
	};

	this.getCaseModel = function() {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetCaseClassInfoRequest();
		request.includeLabels = true;
		request.includeAttributes = true;
		request.includeGlobalObjectAttributes = true;
		//request.caseClasses = "Cases.Servicing.Complaint";
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getGlobaldataAdminService().getCaseClassInfo(request, new callback(deferred));
		return deferred.promise;
	};

	this.readCaseAPI = function(caseRef) {
		//console.log("Called readCase with: " + caseRef);
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ReadCaseRequest(caseRef);
		RestApiService.setRestServiceConfig(restServiceConfig);
		//console.log("Calling readCase with: " + caseRef);
		RestApiService.getGlobaldataService().readCase(request, new callback(deferred));
		return deferred.promise;
	};

	this.listCaseAudit = function(globalDataRef, sortOrder) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetSearchCaseReportRequest(globalDataRef);
		request.sortOrder=sortOrder;
		request.requireallattributes=true;
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getAuditService().getSearchCaseReport(request, new callback(deferred));
		return deferred.promise;
	};


	this.getWorkListItemsForGlobalData = function(globalDataRef, sortOrder) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetWorkListItemsForGlobalDataRequest(0, 100, globalDataRef);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkListService().getWorkListItemsForGlobalData(request, new callback(deferred));
		return deferred.promise;
	};

	this.getWorkListItemsForView = function(vid, startPos, numItems) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetWorkListItemsForViewRequest(vid, startPos, numItems);
		request.getAllocated = false;
		request.enhancedAttributes = true;
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkListService().getWorkListItemsForView(request, new callback(deferred));
		return deferred.promise;
	}

    this.getCaseReferencesForViewByName = function(viewName) {
        var deferred = $q.defer();
        var request = new tibco.objectapi.service.request.GetCaseReferencesForDataViewByNameRequest(viewName);
        RestApiService.setRestServiceConfig(restServiceConfig);
        RestApiService.getGlobaldataService().getCaseReferencesForDataViewByName(request, new callback(deferred));
        return deferred.promise;
    }


    this.rescheduleWorkItemCalendarExtend = function(wid, end) {
		var deferred = $q.defer();
		var workitemSchedule = new tibco.objectapi.brm.RescheduleWorkItem();
		var itemBody = new tibco.objectapi.brm.ItemBody();
		itemParameter = new tibco.objectapi.brm.ParameterType;
		itemParameter.Name = "End";
		itemParameter.Value = end.format();
		itemBody.parameter = [itemParameter];
		workitemSchedule.itemBody = itemBody;
		var request = new tibco.objectapi.service.request.RescheduleWorkItemRequest(workitemSchedule, wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().rescheduleWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.rescheduleWorkItemCalendarMove = function(wid, start, end) {
		var deferred = $q.defer();
		var workitemSchedule = new tibco.objectapi.brm.RescheduleWorkItem();
		var itemBody = new tibco.objectapi.brm.ItemBody();
		itemParameterStart = new tibco.objectapi.brm.ParameterType;
		itemParameterStart.Name = "Start";
		itemParameterStart.Value = start.format();
		itemParameterEnd = new tibco.objectapi.brm.ParameterType;
		itemParameterEnd.Name = "End";
		itemParameterEnd.Value = end.format();
		itemBody.parameter = [itemParameterStart, itemParameterEnd];
		workitemSchedule.itemBody = itemBody;
		var request = new tibco.objectapi.service.request.RescheduleWorkItemRequest(workitemSchedule, wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().rescheduleWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.rescheduleWorkItemCalendarReassign = function(wid, resourceName, resourceGUID) {
		var deferred = $q.defer();
		var workitemSchedule = new tibco.objectapi.brm.RescheduleWorkItem();
		var itemBody = new tibco.objectapi.brm.ItemBody();
		itemParameterResourceName = new tibco.objectapi.brm.ParameterType;
		itemParameterResourceName.Name = "resourceName";
		itemParameterResourceName.Value = resourceName;
		itemParameterResourceGUID = new tibco.objectapi.brm.ParameterType;
		itemParameterResourceGUID.Name = "userGUID";
		itemParameterResourceGUID.Value = resourceGUID;
		itemBody.parameter = [itemParameterResourceName, itemParameterResourceGUID];
		workitemSchedule.itemBody = itemBody;
		var request = new tibco.objectapi.service.request.RescheduleWorkItemRequest(workitemSchedule, wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().rescheduleWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.getOrgModelEntity = function(modelVersion, entityGuids) {
		//console.log('called with: ' + modelVersion, + ' ' + entityGuids);
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetOrgModelEntityRequest(modelVersion);
		request.guids = entityGuids;
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getOrgModelService().getOrgModelEntity(request, new callback(deferred));
		return deferred.promise;
	}

	this.getOfferSet = function(wid) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetOfferSetRequest(wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().getOfferSet(request, new callback(deferred));
		return deferred.promise;
	}

	this.getResourcesForEntity = function(entityGUID) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.FindByEntityRequest(-1, entityGUID);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getOrgResourceService().findByEntity(request, new callback(deferred));
		return deferred.promise;
	}

	this.reallocateTask = function(wid, resourceGUID) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ReallocateWorkItemRequest();
		request.ids = [wid];
		request.resources = [resourceGUID];
		request.revertDatas = [false];
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().reallocateWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.sendEvent = function() {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.EventRequest();
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getAuditService().event(request, new callback(deferred));
		return deferred.promise;
	}

	this.logCaseView = function(caseRef, userId, userName) {
        var deferred = $q.defer();
		if (caseRef!= null && userId != null & userName != null) {

			var eventBatch = new tibco.objectapi.ec.base.EventBatch();
			var event1 = new tibco.objectapi.ec.base.Event();

			event1.eventTimestamp = new Date().toISOString();
			event1.message = "Case Accessed";
			event1.messageId = "CMF_CASE_ACCESS";
			event1.priority = "LOW";
			event1.severity = "AUDIT";
			event1.correlationId = caseRef;

			// convert caseRefString: eg: BDS-1-com.example.casetests.working_case-248-0 to 248:com.example.casetests.working_case:1:BDS
			var caseRefString = caseRef;
			var seploc = caseRefString.indexOf('-');
			var BDSHeader = caseRefString.slice(0, seploc);
			caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
			var seploc = caseRefString.indexOf('-');
			var MajorVers = caseRefString.slice(0, seploc);
			caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
			seploc = caseRefString.indexOf('-');
			var ClassName = caseRefString.slice(0, seploc);
			caseRefString = caseRefString.substring(seploc + 1, caseRefString.length);
			seploc = caseRefString.indexOf('-');
			var BDSId = caseRefString.slice(0, seploc);

			// now we have to reverse the ClassName
			var ClassNameOrg = ClassName;
			var ClassNameRev = ClassName.split('.').reverse().join('.');
			////console.log("*&* Classname: ", ClassNameRev);

			// now create the caseTag
			var caseTag = BDSId + ':' + ClassNameRev + ':' + MajorVers + ':' + BDSHeader;

			var principalNameAttr = {};
			principalNameAttr.attributeName = "principalName";
			principalNameAttr.value = userName;

			var principalIdAttr = {};
			principalIdAttr.attributeName = "principalId";
			principalIdAttr.value = userId;

			var caseSearchTagAttr = {};
			caseSearchTagAttr.attributeName = "caseSearchTag";
			caseSearchTagAttr.value = caseTag;

			var caseMessageCategoryAttr = {};
			caseMessageCategoryAttr.attributeName = "messageCategory";
			caseMessageCategoryAttr.value = "GLOBAL_DATA";

			var managedObjectIdAttr = {};
			managedObjectIdAttr.attributeName = "managedObjectId";
			managedObjectIdAttr.value = caseRef;

			var managedObjectTypeAttr = {};
			managedObjectTypeAttr.attributeName = "managedObjectType";
			managedObjectTypeAttr.value = 'BDS';

			var componentIdAttr = {};
			componentIdAttr.attributeName = "componentId";
			componentIdAttr.value = 'BDS';

			event1.attribute = [principalNameAttr, principalIdAttr, caseSearchTagAttr, caseMessageCategoryAttr, managedObjectIdAttr, managedObjectTypeAttr, componentIdAttr];

			eventBatch.event = [event1];
			////console.log("*&* Called done");
			var request = new tibco.objectapi.service.request.EventRequest(eventBatch);
			////console.log("created request", JSON.stringify(request));
			RestApiService.setRestServiceConfig(restServiceConfig);
			////console.log("*&* sending logCaseView event");
			RestApiService.getAuditService().event(request, new callback(deferred));
			////console.log("*&* sent logCaseView event");
			return deferred.promise;
    	} else {

			var error =  ("Error - tried to post EC message with null parameter: caseRef=" + caseRef + ",userId=" + userId + ",userName=" + userName);
			console.error("*** " + error);
            deferred.reject(error);
            return deferred.promise;
		}
	}

	this.getWorkItemHeader = function (taskId) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetWorkItemHeaderRequest();
		request.ids= [taskId];
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().getWorkItemHeader(request, new callback(deferred));
		return deferred.promise;
	};


	this.logCaseTaskAction = function(caseRef, caseState, userId, userName, eventName, taskName) {
		//console.log("*&* Called logCaseTaskAction " + taskName);
		var action = eventName.slice(3);
		var actionNice;
		//console.log("*&* Sending logCaseTaskAction ", action);
		var deferred = $q.defer();

		////console.log("*&* Called start");
		var eventBatch = new tibco.objectapi.ec.base.EventBatch();
		var event1 =  new tibco.objectapi.ec.base.Event();

		event1.eventTimestamp = new Date().toISOString();
		if (action == 'CLOSE') {
			actionNice = 'closed';
			event1.message = "Case Task Closed";
		}
		if (action == 'OPEN') {
			actionNice = 'opened';
			event1.message = "Case Task Opened";
		}
		if (action == 'CANCEL') {
			actionNice = 'cancelled';
			event1.message = "Case Task Cancelled";
		}
		if (action == 'COMPLETE') {
			actionNice = 'completed';
			event1.message = "Case Task Completed";
		}

		event1.messageId = "CMF_CASE_TASK_" + action;
		event1.priority = "LOW";
		event1.severity = "AUDIT";
		event1.correlationId = caseRef;

		// convert caseRefString: eg: BDS-1-com.example.casetests.working_case-248-0 to 248:com.example.casetests.working_case:1:BDS
		var caseRefString = caseRef;
		var seploc = caseRefString.indexOf('-');
		var BDSHeader = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		var seploc = caseRefString.indexOf('-');
		var MajorVers = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		seploc = caseRefString.indexOf('-');
		var ClassName = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		seploc = caseRefString.indexOf('-');
		var BDSId = caseRefString.slice(0,seploc);

		// now we have to reverse the ClassName
		var ClassNameOrg = ClassName;
		var ClassNameRev = ClassName.split('.').reverse().join('.');
		////console.log("*&* Classname: ", ClassNameRev);

		// now create the caseTag
		var caseTag = BDSId + ':' + ClassNameRev + ':' + MajorVers + ':' + BDSHeader;

		var stateAttr = {};
		stateAttr.attributeName = "managedObjectStatus";
		stateAttr.value = caseState;

		var principalNameAttr = {};
		principalNameAttr.attributeName = "principalName";
		principalNameAttr.value = userName;

		var principalIdAttr = {};
		principalIdAttr.attributeName = "principalId";
		principalIdAttr.value = userId;

		// put task name here
		var processNameAttr = {};
		processNameAttr.attributeName = "processName";
		processNameAttr.value = "Task " + taskName + " " + actionNice;

		// put task name here (attribute for reporting)
		var taskNameAttr = {};
		taskNameAttr.attributeName = "activityId";
		taskNameAttr.value = taskName;

		var caseSearchTagAttr = {};
		caseSearchTagAttr.attributeName = "caseSearchTag";
		caseSearchTagAttr.value = caseTag;

		var caseMessageCategoryAttr = {};
		caseMessageCategoryAttr.attributeName = "messageCategory";
		caseMessageCategoryAttr.value = "GLOBAL_DATA";

		var managedObjectIdAttr = {};
		managedObjectIdAttr.attributeName = "managedObjectId";
		managedObjectIdAttr.value = caseRef;

		var managedObjectTypeAttr = {};
		managedObjectTypeAttr.attributeName = "managedObjectType";
		managedObjectTypeAttr.value = 'BDS';

		var componentIdAttr = {};
		componentIdAttr.attributeName = "componentId";
		componentIdAttr.value = 'BDS';

		event1.attribute = [principalNameAttr, principalIdAttr, caseSearchTagAttr, caseMessageCategoryAttr, managedObjectIdAttr, managedObjectTypeAttr, componentIdAttr, processNameAttr, stateAttr, taskNameAttr];

		eventBatch.event = [event1];
		////console.log("*&* Called done");
		var request = new tibco.objectapi.service.request.EventRequest(eventBatch);
		////console.log("created request", JSON.stringify(request));
		RestApiService.setRestServiceConfig(restServiceConfig);
		////console.log("*&* sending logCaseView event");
		RestApiService.getAuditService().event(request, new callback(deferred));
		//console.log("*&* sent logCaseView event");

		return deferred.promise;

	}

		this.logCaseAudit = function(caseRef, userName, userId, processName, state, messageId, message) {
		//console.log("*&* Called logCaseAudit ", messageId);
		var deferred = $q.defer();
		////console.log("*&* Called start");
		var eventBatch = new tibco.objectapi.ec.base.EventBatch();
		var event1 =  new tibco.objectapi.ec.base.Event();

		event1.eventTimestamp = new Date().toISOString();
        event1.message = message;
        event1.messageId = messageId;
        event1.priority = "LOW";
        event1.severity = "AUDIT";
		event1.correlationId = caseRef;

		// convert caseRefString: eg: BDS-1-com.example.casetests.working_case-248-0 to 248:com.example.casetests.working_case:1:BDS
		var caseRefString = caseRef;
		var seploc = caseRefString.indexOf('-');
		var BDSHeader = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		var seploc = caseRefString.indexOf('-');
		var MajorVers = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		seploc = caseRefString.indexOf('-');
		var ClassName = caseRefString.slice(0,seploc);
		caseRefString = caseRefString.substring(seploc+1,caseRefString.length);
		seploc = caseRefString.indexOf('-');
		var BDSId = caseRefString.slice(0,seploc);

		// now we have to reverse the ClassName
		var ClassNameOrg = ClassName;
		var ClassNameRev = ClassName.split('.').reverse().join('.');
		////console.log("*&* Classname: ", ClassNameRev);

		// now create the caseTag
		var caseTag = BDSId + ':' + ClassNameRev + ':' + MajorVers + ':' + BDSHeader;


		var stateAttr = {};
		stateAttr.attributeName = "managedObjectStatus";
		stateAttr.value = state;

		var processNameAttr = {};
		processNameAttr.attributeName = "processName";
		processNameAttr.value = processName;


		var principalNameAttr = {};
		principalNameAttr.attributeName = "principalName";
		principalNameAttr.value = userName;

		var principalIdAttr = {};
		principalIdAttr.attributeName = "principalId";
		principalIdAttr.value = userId;

		var caseSearchTagAttr = {};
		caseSearchTagAttr.attributeName = "caseSearchTag";
		caseSearchTagAttr.value = caseTag;

		var caseMessageCategoryAttr = {};
		caseMessageCategoryAttr.attributeName = "messageCategory";
		caseMessageCategoryAttr.value = "GLOBAL_DATA";

		var managedObjectIdAttr = {};
		managedObjectIdAttr.attributeName = "managedObjectId";
		managedObjectIdAttr.value = caseRef;

		var managedObjectTypeAttr = {};
		managedObjectTypeAttr.attributeName = "managedObjectType";
		managedObjectTypeAttr.value = 'BDS';

		var componentIdAttr = {};
		componentIdAttr.attributeName = "componentId";
		componentIdAttr.value = 'BDS';

		event1.attribute = [stateAttr, processNameAttr, principalNameAttr, principalIdAttr, caseSearchTagAttr, caseMessageCategoryAttr, managedObjectIdAttr, managedObjectTypeAttr, componentIdAttr];

		eventBatch.event = [event1];
		////console.log("*&* Called done");
		var request = new tibco.objectapi.service.request.EventRequest(eventBatch);
		////console.log("created request", JSON.stringify(request));
		RestApiService.setRestServiceConfig(restServiceConfig);
		////console.log("*&* sending logCaseAudit event");
		RestApiService.getAuditService().event(request, new callback(deferred));
		////console.log("*&* sent logCaseAudit event");
		return deferred.promise;

	}

	this.rescheduleWorkItemCalendarReassign = function(wid, resourceName, resourceGUID) {
		var deferred = $q.defer();
		var workitemSchedule = new tibco.objectapi.brm.RescheduleWorkItem();
		var itemBody = new tibco.objectapi.brm.ItemBody();
		itemParameterResourceName = new tibco.objectapi.brm.ParameterType;
		itemParameterResourceName.Name = "resourceName";
		itemParameterResourceName.Value = resourceName;
		itemParameterResourceGUID = new tibco.objectapi.brm.ParameterType;
		itemParameterResourceGUID.Name = "userGUID";
		itemParameterResourceGUID.Value = resourceGUID;
		itemBody.parameter = [itemParameterResourceName, itemParameterResourceGUID];
		workitemSchedule.itemBody = itemBody;
		var request = new tibco.objectapi.service.request.RescheduleWorkItemRequest(workitemSchedule, wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().rescheduleWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.getOfferSet = function(wid) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetOfferSetRequest(wid);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().getOfferSet(request, new callback(deferred));
		return deferred.promise;
	}

	this.getResourcesForEntity = function(entityGUID) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.FindByEntityRequest(-1, entityGUID);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getOrgResourceService().findByEntity(request, new callback(deferred));
		return deferred.promise;
	}

	this.reallocateTask = function(wid, resourceGUID) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ReallocateWorkItemRequest();
		request.ids = [wid];
		request.resources = [resourceGUID];
		request.revertDatas = [false];
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().reallocateWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

	this.reallocateTask = function(wid, resourceGUID) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.ReallocateWorkItemRequest();
		request.ids = [wid];
		request.resources = [resourceGUID];
		request.revertDatas = [false];
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkItemManagementService().reallocateWorkItem(request, new callback(deferred));
		return deferred.promise;
	}

    this.previewWorkItemsFromList = function (taskIds, guid) {
        var deferred = $q.defer();
        var request = new tibco.objectapi.service.request.PreviewWorkItemFromListRequest("RESOURCE",guid);
        request.ids = [];
        request.versions = [];
        for (x=0;x<taskIds.length;x++) {
			request.ids.push(taskIds[x]);
            request.versions.push(-1);
		}
        RestApiService.setRestServiceConfig(restServiceConfig);
        RestApiService.getWorkListService().previewWorkItemFromList(request, new callback(deferred));
        return deferred.promise;
    };

	this.previewWorkItemFromList = function (taskId, guid) {
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.PreviewWorkItemFromListRequest("RESOURCE",guid);
		request.ids= [taskId];
		request.versions= [-1];
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkListService().previewWorkItemFromList(request, new callback(deferred));
		return deferred.promise;
	};

	this.getViewDetails = function (viewId) {
		//console.log("*grid - getting details for view: " + viewId);
		var deferred = $q.defer();
		var request = new tibco.objectapi.service.request.GetWorkListViewDetailsRequest(viewId, false);
		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getWorkListService().getWorkListViewDetails(request, new callback(deferred));
		return deferred.promise;
	};

    this.queryBusinessServicesByCategory = function (category) {
        var deferred = $q.defer();
        var request = new tibco.objectapi.bs.base.QueryBusinessServicesRequest();
        request.category = category;
        request.includeFormalParameters = true;
        RestApiService.setRestServiceConfig(restServiceConfig);
        RestApiService.getBusinessServiceService().queryBusinessServices(request, new callback(deferred));
        return deferred.promise;
    };

	// unfortunately we have to make a direct REST call due to objectApi not returning the CDATA for columns in JSON
	this.getViewColumnDetails = function (viewId) {
		var deferred = $q.defer();

		var serviceUrl = getBPMRestUrl() + 'worklist/worklistview/' + viewId + '/false';
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'POST',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/xml'
			},
			data: { }
		};
		$http(req).then(function(response) {
			    var data = response.data;
				// extract the view definition XML segment
				var xmlDoc = $.parseXML( data );
				$xml = $( xmlDoc );
				var clientViewXML = $xml.find( "customData" );
				var clientViewText = clientViewXML.text();

				// now convert clientView to JSON
				// Call the xml2json function
				var clientViewJSON = xml2json(clientViewText);

				// finally send back the data
				deferred.resolve(clientViewJSON);
			})
		return deferred.promise;
	};

	this.persistWlColumnSettings = function (guid,columns) {

		var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/' + 'CMF_WorkListConfig?columns=' + columns;
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'PUT',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/json'
			},
			data: { }
		};
		$http(req).then(function(response) {
		})
		return;
	};

	this.recordCaseAccess = function (guid,caseRef) {
		var shortCaseRef = caseRef.slice(0,caseRef.lastIndexOf("-"));
		var shortCaseRef = shortCaseRef + '-0';
		var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/' + 'CMF_CaseAccess_' + shortCaseRef + '?date=' + new Date() + '&caseRef=' +caseRef;
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'PUT',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/json'
			},
			data: { }
		};
		$http(req).then(function(response) {
		})
		return;
	};

	this.retrieveCaseAccess = function (guid, count) {
		var deferred = $q.defer();
		var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid;
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'GET',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/json'
			},
			data: { }
		};
		$http(req).then(function(response) {
			var data = response.data;
			// filter list
			var cases = [];
			var settings=data['xml-fragment'].settingId;
			for (x=0; x< settings.length; x++) {
				setting = settings[x];
				if (setting.slice(0, 14) === 'CMF_CaseAccess' ) {
					var newCase = { caseRef: setting.slice(15) , lastAccess: Date() }
					cases.push(newCase);
				}
			}

			// just return last <count> cases refs
			cases = cases.slice(0).slice(-count);
			deferred.resolve(cases);
		})
		return deferred.promise;
	};

	this.retrieveCaseAccessEC = function (count) {
		var deferred = $q.defer();
		var sortElement = {};
		sortElement.attribute = 'creationTime';
		sortElement.type = 'DESCENDING';

		var query = new tibco.objectapi.ec.base.Query();
		query.sortOrder = [ sortElement ];
		query.filter = "messageCategory='GLOBAL_DATA' AND messageId='CMF_CASE_ACCESS' AND currentUser(principalName)";

		var queryPayload = new tibco.objectapi.ec.base.QueryPayload();
		queryPayload.query=(query);


		var request = new tibco.objectapi.service.request.ExecuteQueryAuditRequest(queryPayload);
		request.numberOfItems = count;
		//request.requireAllAttributes='true';
		request.requiredAttributes=['managedObjectId', 'creationTime'];
		request.target='AUDIT';

		RestApiService.setRestServiceConfig(restServiceConfig);
		RestApiService.getAuditService().executeQuery(request, new callback(deferred));

		return deferred.promise;

	};

	this.readCaseFavourites = function (guid) {
		var deferred = $q.defer();
		var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/CMF_CaseFavourites';
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'GET',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/json'
			},
			data: { }
		};
		$http(req).then(function(response) {
			var data = response.data;
			deferred.resolve(data);
		})
		return deferred.promise;
	};

	this.addToCaseFavourites = function (guid,caseRef) {

		var deferred = $q.defer();
		var shortCaseRef = caseRef.slice(0,caseRef.lastIndexOf("-"));
		var shortCaseRef = shortCaseRef + '-0';
		var favouriteCaseRefs = [];
		var favouriteCaseRefsString;
		// first read favourites
		this.readCaseFavourites(guid).then(
				function(result) {
					settingProperties = result['settingProperties'];
					if (settingProperties == undefined) {
						// no existing favourites so just add this one
						favouriteCaseRefs.push(shortCaseRef);
					} else {
						// append to existing favourites
						// is this single item array?
						if (settingProperties['@Value'] != undefined) {
                            favouriteCaseRefs = settingProperties['@Value'].split(',');
                        } else {
                            favouriteCaseRefs = settingProperties[0]['@Value'].split(',');
						}
                        favouriteCaseRefs.push(shortCaseRef);
					}
					favouriteCaseRefsString = favouriteCaseRefs.join(',');

					// now save the updated favourites
					var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/CMF_CaseFavourites?caseRefs=' + favouriteCaseRefsString;
					////console.log("resturl is: " + serviceUrl);
					var reqInner = {
						method: 'PUT',
						url: serviceUrl,
						headers: {
							'Content-Type': 'application/json',
							'accept': 'application/json'
						},
						data: { }
					};
					$http(reqInner).then(function(response) {
						var data = response.data;
						deferred.resolve(data);
					})
				}
		)
		return deferred.promise;
	};

	this.removeFromCaseFavourites = function (guid,caseRef) {

		var deferred = $q.defer();
		var shortCaseRef = caseRef.slice(0,caseRef.lastIndexOf("-"));
		var shortCaseRef = shortCaseRef + '-0';
		var favouriteCaseRefs = [];
		var favouriteCaseRefsString;
		// first read favourites
		this.readCaseFavourites(guid).then(
				function(result) {
					settingProperties = result['settingProperties'];
                    if (settingProperties['@Value'] != undefined) {
                        favouriteCaseRefs = settingProperties['@Value'].split(',');
                    } else {
                        favouriteCaseRefs = settingProperties[0]['@Value'].split(',');
					}
					// remove from existing favourites
					var loc = favouriteCaseRefs.indexOf(shortCaseRef);
					favouriteCaseRefs.splice(loc,1);
					favouriteCaseRefsString = favouriteCaseRefs.join(',');
					// now save the updated favourites
					var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/CMF_CaseFavourites?caseRefs=' + favouriteCaseRefsString;
					////console.log("resturl is: " + serviceUrl);
					var reqInner = {
						method: 'PUT',
						url: serviceUrl,
						headers: {
							'Content-Type': 'application/json',
							'accept': 'application/json'
						},
						data: { }
					};
					$http(reqInner).then(function(response) {
						var data = response.data;
						deferred.resolve(data);
					})
				}
		)
		return deferred.promise;
	};

	this.readWlColumnSettings = function (guid) {

		var deferred = $q.defer();
		var serviceUrl = getBPMRestUrl() + 'usersettings/' + guid + '/' + 'CMF_WorkListConfig';
		////console.log("resturl is: " + serviceUrl);
		var req = {
			method: 'GET',
			url: serviceUrl,
			headers: {
				'Content-Type': 'application/json',
				'accept': 'application/json'
			},
			data: { }
		};
		$http(req).then(function(response) {
			var data = response.data;
			// extract the column data
			var settingProperties = data['settingProperties'];
			if (settingProperties != undefined) {
				if (settingProperties['@Name'] == 'columns') {
					// create array
					response = settingProperties['@Value'].split(',');
				}
			} else {
				// nothing set so respond with empty string
				response = '';
			}
			deferred.resolve(response);
		})
		return deferred.promise;
	};




});